/*
  Provede založení dokladové řady pro rok, pokud dokladová řada pro rok a danou řadu ještě neexistuje.

  Vstup:
  - i_idRady          - id řady
  - i_idUcetniObdobi  - id účetního období, pro které se zakládá řada

  Autor: Pavel Cvrček
*/

create or replace function dZalozRaduRok (in i_idRady integer, in i_idUcetniObdobi integer, in i_modul varchar(3)) returns VOID as $$
declare
  _PlatiOdData date;
  _idUcetObdobi integer;
  _idNalezeneRady integer;  
  
  _CisDelka integer;
  _ZobrazNuly boolean;
  _CisAkt integer;
  _CisPoc integer;
  _Prefix varchar;
  _Postfix varchar;
  _idRadyRok integer;
  _Unique boolean;
  _PostfixIncrement integer;   
  _PocetNalezeni integer;
  i integer;
  _DefPostfix varchar;
  _idRady integer;
   
  _radyCursor refcursor;
begin

  /* Zjistí se, zda pro starší účetní období existuje položka dokladové řady. */  
  select platiOdData into _PlatiOdData from nUcetObdobi where iducetobdobi = i_idUcetniObdobi;
  select idUcetObdobi into _idUcetObdobi from nUcetObdobi where platiOdData < _PlatiOdData order by platiOdData desc limit 1;
  if (_idUcetObdobi is not null) then 
    select idRadyRok into _idNalezeneRady from dRadyRok where idRady = i_idRady and iducetobdobi = _idUcetObdobi;
  end if;

  /* Pokud byla nějaká dokladová řada pro starší rok nalezena, provede se překopírování záznamu. */
  if (_idNalezeneRady is not null) then
    
    /* Vrátí se data z nejnovějšího roku */
    select CisDelka, ZobrazNuly, CisAkt, CisPoc, Prefix, Postfix, idRadyRok into _CisDelka,_ZobrazNuly,_CisAkt,_CisPoc,_Prefix,_Postfix,
    _idRadyRok from dRadyRok where idRady = i_idRady and iducetobdobi = _idUcetObdobi;
      
      
    /* Provede se vložení záznamu pro nejvyšší rok */
    insert into dRadyRok (idRady,idUcetObdobi,CisDelka,ZobrazNuly,CisAkt,CisPoc,Prefix,Postfix,LastUpdate) values
          (i_idRady, i_idUcetniObdobi, _CisDelka, _ZobrazNuly, _CisPoc, _CisPoc, _Prefix, _Postfix, now());
  
  else
    /* Neexistuje žádná položka dokladové řady pro starší účetní období, vytvoří se nová s implicitními hodnotami.
       Provede se též kontrola, zda nově vytvořený záznam dokladové řady pro daný rok nekoliduje s již existujícím záznamem.
       Pokud ano, provede se úprava výchozích hodnot řady tak, aby byla nově přidaná dokladová řada pro rok unikátní. */
    
    _defPostfix:= '/%YY%';
    _Unique:= false;
    _postfixIncrement:= 0;
    _CisPoc:= 1;   
    
    while (_Unique = false) loop
      _Postfix:= _DefPostfix;
      if (_PostfixIncrement > 0) then
        _Postfix:= _postfix || '-' || cast (_PostfixIncrement as text);
      end if;      
      
      _PocetNalezeni:= 0;
      open _RadyCursor for select idRady from dRady where modul = i_modul;
      loop
        fetch _Radycursor into i;
        if (i is null) then
          exit;
        end if;
        
        select idRady into _idRady from dRadyRok where Prefix = '' and Postfix = _postfix and idUcetObdobi = i_idUcetniObdobi and idRady = i and cispoc = _cisPoc;
        if (_idRady is not null) then
          _PocetNalezeni:= _PocetNalezeni + 1;
        end if;                
      end loop;
      close _RadyCursor;
      
      
      if _PocetNalezeni = 0 then
        _Unique:= true;
      end if;
      
      _PostfixIncrement:= _PostfixIncrement + 1;       
    end loop;           

    insert into dRadyRok (idRady,idUcetObdobi,CisDelka,ZobrazNuly,CisAkt,CisPoc,Prefix,Postfix,LastUpdate) values
      (i_idRady,i_idUcetniObdobi,8,true,_cisPoc,_cisPoc,'',_postfix,now());
       
  end if;  
end;
$$ LANGUAGE plpgsql;