CREATE OR REPLACE FUNCTION logzurnal() RETURNS trigger AS $$
    # PL/Tcl function body

    spi_exec "SELECT pg_attribute.attname AS pk
        FROM pg_index, pg_class, pg_attribute
        WHERE
          pg_class.oid = '[ quote $TG_table_name ]'::regclass AND
          indrelid = pg_class.oid AND
          pg_attribute.attrelid = pg_class.oid AND
          pg_attribute.attnum = any(pg_index.indkey)
          AND indisprimary"

    if {$TG_op == "INSERT" || $TG_op == "UPDATE"} {
        set pkval [ quote $NEW([ quote $pk ]) ]
    } else {
        set pkval [ quote $OLD([ quote $pk ]) ]
    }

    spi_exec "SELECT now() AS log_time"

    spi_exec "SELECT getGloTxVariable('zurnalTxName') AS tx_name"
    if {![ info exists tx_name ]} {
        set tx_name null
    } else {
        set tx_name '[ quote $tx_name ]'
    }

    spi_exec "SELECT session_user AS user_name"
    set user_dollar_idx [string first \$ $user_name] 
    if {$user_dollar_idx != -1} {
        incr user_dollar_idx
        set user_name "[string range $user_name $user_dollar_idx end]" 
    }

    if {$TG_op == "UPDATE" || $TG_op == "DELETE"} {

      spi_exec -array C "select attname from pg_attribute where attrelid = $TG_relid and attnum > 0 and not attisdropped" {
          if {$C(attname) == "lastupdate"} {
              continue;
          }

          if {[ info exists NEW([ quote $C(attname) ]) ]} {
              if {[ info exists OLD([ quote $C(attname) ]) ]} {
                  if {$NEW([ quote $C(attname) ]) != $OLD([ quote $C(attname) ])} {
                      spi_exec "INSERT INTO wzurnal(datcas,uziv,tabulka,sloupec,idzaznamu,novahod,starahod,op,transakcek)
                          VALUES ('[ quote $log_time ]',
                              '[ quote $user_name ]',
                              '[ quote $TG_table_name ]',
                              '[ quote $C(attname) ]',
                              '[ quote $pkval ]',
                              '[ quote $NEW([ quote $C(attname) ]) ]',
                              '[ quote $OLD([ quote $C(attname) ]) ]',
                              '[ quote $TG_op ]',
                              $tx_name)"
                  }
              } else {
                  spi_exec "INSERT INTO wzurnal(datcas,uziv,tabulka,sloupec,idzaznamu,novahod,starahod,op,transakcek)
                      VALUES ('[ quote $log_time ]',
                          '[ quote $user_name ]',
                          '[ quote $TG_table_name ]',
                          '[ quote $C(attname) ]',
                          '[ quote $pkval ]',
                          '[ quote $NEW([ quote $C(attname) ]) ]',
                          null,
                          '[ quote $TG_op ]',
                          $tx_name)"
              }
          } else {
              if {[ info exists OLD([ quote $C(attname) ]) ]} {
                  spi_exec "INSERT INTO wzurnal(datcas,uziv,tabulka,sloupec,idzaznamu,novahod,starahod,op,transakcek)
                      VALUES ('[ quote $log_time ]',
                          '[ quote $user_name ]',
                          '[ quote $TG_table_name ]',
                          '[ quote $C(attname) ]',
                          '[ quote $pkval ]',
                          null,
                          '[ quote $OLD([ quote $C(attname) ]) ]',
                          '[ quote $TG_op ]',
                          $tx_name)"
              } else {
                  # no change -> do nothing
              }
          }
      }

    } else {
        if {$TG_op == "INSERT"} {
              spi_exec "INSERT INTO wzurnal(datcas,uziv,tabulka,sloupec,idzaznamu,novahod,starahod,op,transakcek)
                  VALUES ('[ quote $log_time ]',
                      '[ quote $user_name ]',
                      '[ quote $TG_table_name ]',
                      null,
                      '[ quote $pkval ]',
                      null,
                      null,
                      '[ quote $TG_op ]',
                      $tx_name)"
        }
    }


    if {$TG_op == "DELETE"} {
        return [array get OLD]
    } else {
        return [array get NEW]
    }
$$ LANGUAGE pltcl VOLATILE SECURITY DEFINER;

ALTER FUNCTION public.logzurnal() OWNER TO %admin%;
