--------------------------------------------------------------------------------
-- Bug 21742 – [WinStrom] Detekce firem s rozbitým účetnictvím
--------------------------------------------------------------------------------

create or replace function overchyby(_dotaz varchar, tabulka varchar, pole varchar, poleKop varchar) returns varchar as $$
declare
  chyb integer;
  dotaz varchar;
begin
  dotaz = replace(_dotaz, '%table%', tabulka);
  dotaz = replace(dotaz, '%pole%', pole);
  dotaz = replace(dotaz, '%koppole%', poleKop);
  
  execute dotaz into chyb;
  
  if (chyb > 0) then
    return 'Prenos doklad/polozka tabulka: dpol' || tabulka || ' vlastnost: ' || pole || ' chyb: ' || chyb || E'\n';
  end if;
  
  return '';
end;
$$ LANGUAGE plpgsql;
ALTER FUNCTION overchyby(varchar, varchar, varchar, varchar) OWNER TO %firmaRole%;

create or replace function runtest() returns void as $$
declare
  vysledek varchar;
  chyb integer;
  tabulky varchar[];
  tabulkyUcty varchar[];
  tabulka int;
  poleProp varchar[];
  poleKop varchar[];
  pole integer;
  dotaz varchar;
begin
  vysledek = '';
  -- tabulky pole, ktere se kopiruji z hlavicky
  tabulky = array['fak', 'int', 'sklad', 'obch'];
  tabulkyUcty = array['fak', 'sklad'];
  poleProp = array['idtypucop', 'idclendph', 'idstred', 'idcinnost', 'idzakazky'];
  poleKop = array['koptypucop', 'kopclendph', 'kopstred', 'kopcinnost', 'kopzakazku'];

  -- kontrola zaskrtavatek kopirovat z hlavicky
  FOR tabulka IN 1 .. array_upper(tabulkyUcty, 1) LOOP
    dotaz = 'select count(*) from dpol%table% as P left outer join ddokl%table% as D on P.iddokl%table% = D.iddokl%table% ' ||
      'where P.kopzklmducet and ((D.typpohybuk = ''typPohybu.vydej'' and compareid(P.idzklmducet, D.idprotiucet) = false) or ' ||
                                '(D.typpohybuk = ''typPohybu.prijem'' and compareid(P.idzklmducet, D.idprimucet) = false))';
    vysledek = vysledek || overchyby(dotaz, tabulkyUcty[tabulka], 'idzklmducet', 'kopzklmducet');

    dotaz = 'select count(*) from dpol%table% as P left outer join ddokl%table% as D on P.iddokl%table% = D.iddokl%table% ' ||
      'where P.kopzkldalucet and ((D.typpohybuk = ''typPohybu.vydej'' and compareid(P.idzkldalucet, D.idprimucet) = false) or ' ||
                                 '(D.typpohybuk = ''typPohybu.prijem'' and compareid(P.idzkldalucet, D.idprotiucet) = false))';
    vysledek = vysledek || overchyby(dotaz, tabulkyUcty[tabulka], 'idzkldalucet', 'kopzkldalucet');

    dotaz = 'select count(*) from dpol%table% as P left outer join ddokl%table% as D on P.iddokl%table% = D.iddokl%table% ' ||
      'where P.kopdphmducet and P.typszbdphk <> ''typSzbDph.dphOsv'' and ((D.typpohybuk = ''typPohybu.vydej'' and compareid(P.iddphmducet, (' ||
         'case when P.typszbdphk = ''typSzbDph.dphZakl'' then D.iddphzaklucet else ' ||
         '(case when P.typszbdphk = ''typSzbDph.dphSniz'' then D.iddphsnizucet else null end) end)) = false) or ' ||
                                '(D.typpohybuk = ''typPohybu.prijem'' and compareid(P.iddphmducet, D.idprimucet) = false))';
    vysledek = vysledek || overchyby(dotaz, tabulkyUcty[tabulka], 'iddphmducet', 'kopdphmducet');
  
    dotaz = 'select count(*) from dpol%table% as P left outer join ddokl%table% as D on P.iddokl%table% = D.iddokl%table% ' ||
      'where P.kopdphdalucet and P.typszbdphk <> ''typSzbDph.dphOsv'' and ((D.typpohybuk = ''typPohybu.vydej'' and compareid(P.iddphdalucet, D.idprimucet) = false) or ' || 
                                 '(D.typpohybuk = ''typPohybu.prijem'' and compareid(P.iddphdalucet, (' ||
         'case when P.typszbdphk = ''typSzbDph.dphZakl'' then D.iddphzaklucet else ' ||
         '(case when P.typszbdphk = ''typSzbDph.dphSniz'' then D.iddphsnizucet else null end) end)) = false))';
    vysledek = vysledek || overchyby(dotaz, tabulkyUcty[tabulka], 'iddphdalucet', 'kopdphdalucet');
  END LOOP;
  
  dotaz = 'select count(*) from dpol%table% as P left outer join ddokl%table% as D on P.iddokl%table% = D.iddokl%table% ' ||
          '  where P.%koppole% and compareid(P.%pole%, D.%pole%) = false';
  
  FOR tabulka IN 1 .. array_upper(tabulky, 1) LOOP
    FOR pole IN 1 .. array_upper(poleProp, 1) LOOP
      vysledek = vysledek || overchyby(dotaz, tabulky[tabulka], poleProp[pole], poleKop[pole]);
    END LOOP;
  END LOOP;
  
  -- vazba prijemka/sklad
  select count(*) into chyb from dvazebtab as V, dpolfak as F, dpolsklad as S where V.idtaba = F.idpolfak and V.idtabb = S.idpolsklad
    and typvazbyk = 'typVazbyDokl.polozkaSkl' and modula = 'FAP' and compareid(F.idzklmducet, S.idzkldalucet) = false;
  if (chyb > 0) then
    vysledek = vysledek || 'Prenos uctu FAP/sklad: ' || chyb || E'\n';
  end if;
  
  -- vazba pokladna/sklad
  select count(*) into chyb from dvazebtab as V, dpolint as P, ddoklint as D, dpolsklad as S 
      where V.idtaba = P.idpolint and V.idtabb = S.idpolsklad and D.iddoklint = P.iddoklint
        and typvazbyk = 'typVazbyDokl.polozkaSkl' and modula = 'POK' and D.typpohybuk = 'typPohybu.vydej' 
        and compareid(P.idzklmducet, S.idzkldalucet) = false;
  if (chyb > 0) then
    vysledek = vysledek || 'Prenos uctu POK/sklad: ' || chyb || E'\n';
  end if;
  
  -- prevodka
  select count(*) into chyb from dvazebtab as VT, dpolsklad as P, dpolsklad as V where VT.idtaba = V.idpolsklad and VT.idtabb = P.idpolsklad
    and modula = 'SKL' and modulb = 'SKL' and typvazbyk = 'typVazbyDokl.polozkaSklPrevod' and compareid(V.idzklmducet, P.idzkldalucet) = false;
  if (chyb > 0) then
    vysledek = vysledek || 'Prenos uctu prevodka: ' || chyb || E'\n';
  end if;
  
  -- odpocet ZDD - zaklad
  select count(*) into chyb from dvazebtab as V, dpolfak as P, ddoklfak as D where V.idtaba = P.idpolfak and V.idtabb = P.iddoklfak
    and typvazbyk = 'typVazbyDokl.odpocetZdd' and compareid(D.idprotiucet, 
    (case when D.typpohybuk = 'typPohybu.prijem' then P.idzkldalucet else P.idzklmducet end)) = false;
  if (chyb > 0) then
    vysledek = vysledek || 'Prenos uctu odpocet ZDD zaklad: ' || chyb || E'\n';
  end if;
  
  -- odpocet ZDD - dan
  select count(*) into chyb from dvazebtab as V, dpolfak as P, ddoklfak as D where V.idtaba = P.idpolfak and V.idtabb = P.iddoklfak
    and typvazbyk = 'typVazbyDokl.odpocetZdd' and P.typSzbDphK <> 'typSzbDph.dphOsv' and 
    compareid((case when P.typszbdphk = 'typSzbDph.dphSniz' then D.iddphsnizucet else D.iddphzaklucet end), 
    (case when D.typpohybuk = 'typPohybu.prijem' then P.idzkldalucet else P.idzklmducet end)) = false;
  if (chyb > 0) then
    vysledek = vysledek || 'Prenos uctu odpocet ZDD dan: ' || chyb || E'\n';
  end if;
    
  -- vyhozeni chyby
  if (char_length(vysledek) > 0) then
    select coalesce(nazfirmy, '') || ', mail:' || coalesce(email, '') || ', tel:' || coalesce(tel, '')
      from nnastav into dotaz limit 1;
    vysledek = 'Chyby u firmy: ' || dotaz || E'\n'  || vysledek;
  
    raise exception '%', vysledek;
  end if;
end;
$$ LANGUAGE plpgsql;
ALTER FUNCTION runtest() OWNER TO %firmaRole%;

select runtest();

drop function runtest();
drop function overchyby(varchar, varchar, varchar, varchar);
