----------------------------------------------------------------
-- Bug 15222 – [WinStrom] Přílohy k dokladu a jejich viditelnost
----------------------------------------------------------------

alter table aadresar add column pocetPriloh integer not null default 0;
alter table akontakty add column pocetPriloh integer not null default 0;
alter table ccenik add column pocetPriloh integer not null default 0;

alter table ddoklfak add column pocetPriloh integer not null default 0;
alter table ddoklint add column pocetPriloh integer not null default 0;
alter table ddoklobch add column pocetPriloh integer not null default 0;
alter table ddoklsklad add column pocetPriloh integer not null default 0;

create or replace function updatePocetPriloh(prilohaid integer, addon integer) returns void as
$BODY$
declare
    priloha record;
begin
    select * into priloha from wpriloha p where p.idpriloha = prilohaid;

    if (priloha.idadresar is not null) then
        update aadresar 
        set pocetPriloh =   (select count(1)+addon
                            from wpriloha p
                            where p.idadresar = priloha.idadresar)
        where idfirmy = priloha.idadresar;
    end if;

    if (priloha.idkontakt is not null) then
        update akontakty set pocetPriloh =  (select count(1)+addon
                                            from wpriloha p
                                            where p.idkontakt = priloha.idkontakt)
        where idkontakt = priloha.idkontakt;
    end if;

    if (priloha.idcenik is not null) then
        update ccenik set pocetPriloh = (select count(1)+addon
                                        from wpriloha p
                                        where p.idcenik = priloha.idcenik)
        where idcenik = priloha.idcenik;
    end if;

    if (priloha.iddoklfak is not null) then
        update ddoklfak set pocetPriloh =   (select count(1)+addon
                                            from wpriloha p
                                            where p.iddoklfak = priloha.iddoklfak)
        where iddoklfak = priloha.iddoklfak;
    end if;

    if (priloha.iddoklint is not null) then
        update ddoklint set pocetPriloh =   (select count(1)+addon
                                            from wpriloha p
                                            where p.iddoklint = priloha.iddoklint)
        where iddoklint = priloha.iddoklint;
    end if;

    if (priloha.iddoklobch is not null) then
        update ddoklobch set pocetPriloh =  (select count(1)+addon
                                            from wpriloha p
                                            where p.iddoklobch = priloha.iddoklobch)
        where iddoklobch = priloha.iddoklobch;
    end if;

    if (priloha.iddoklsklad is not null) then
        update ddoklsklad set pocetPriloh = (select count(1)+addon
                                            from wpriloha p
                                            where p.iddoklsklad = priloha.iddoklsklad)
        where iddoklsklad = priloha.iddoklsklad;
    end if;
end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;
ALTER FUNCTION updatePocetPriloh(integer, integer) OWNER TO %firmaRole%;

-- triggery

create or replace function insertPocetPriloh() returns trigger as
$BODY$
begin
    perform updatePocetPriloh(NEW.idpriloha, 0);
    return NEW;
end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;
ALTER FUNCTION insertPocetPriloh() OWNER TO %firmaRole%;

create or replace function deletePocetPriloh() returns trigger as
$BODY$
begin
    perform updatePocetPriloh(OLD.idpriloha, -1);
    return OLD;
end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;
ALTER FUNCTION deletePocetPriloh() OWNER TO %firmaRole%;

CREATE TRIGGER wprilohainsertpocetpriloh
  AFTER INSERT ON wpriloha
  FOR EACH ROW
  EXECUTE PROCEDURE insertPocetPriloh();

CREATE TRIGGER wprilohadeletepocetpriloh
  BEFORE DELETE ON wpriloha
  FOR EACH ROW
  EXECUTE PROCEDURE deletePocetPriloh();

-- update aktualniho stavu
create or replace function updateAllPocetPriloh() returns void as
$BODY$
declare
    prilohaid integer;
begin
    for prilohaid in (select idpriloha from wpriloha)
    loop
        perform updatePocetPriloh(prilohaid, 0);
    end loop;
end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;
ALTER FUNCTION updateAllPocetPriloh() OWNER TO %firmaRole%;

select updateAllPocetPriloh();
drop function updateAllPocetPriloh();

-- zvyseni verze databaze
insert into dbversion(iddbversion, dbversion, needreimport)
  values (nextval('seq_dbversion'), 174, 't');