-- Bug 11378 – [WinStrom] Obecné provázání dokladů

-- uprava funkce kontrolující zda je doklad nebo položka ve vazbě
CREATE OR REPLACE FUNCTION dzkontrolujvazebnitab(i_iddokl integer, i_isdoklad boolean, i_modul character varying)
  RETURNS boolean AS $$
declare
  _pocetVazeb integer;
begin
  
  /* Jedná se o položku dokladu */
  if (i_isDoklad = false) then
    /* idtaba i idtabb jsou položky */
    select count(*) into _pocetVazeb from dVazebTab where (typvazbyk in ('typVazbyDokl.polozkaSkl',
                                                             'typVazbyDokl.obchod_obchod_pol', 'typVazbyDokl.obchod_faktura_pol',
                                                             'typVazbyDokl.obchod_zaloha_pol', 'typVazbyDokl.obchod_sklad_pol'))
                                                      and ((idtaba = i_idDokl and modula = i_modul) or (idtabb = i_idDokl and modulb = i_modul));
    if (_PocetVazeb <> 0) then
      return false;
    end if;
    
    /* kontrola, kde je pro položku dokladu jen u idtaba, jedná se o "typVazbyDokl.odpocetZALOHY" a "typVazbyDokl.odpocetZDD" */
    select count(*) into _pocetVazeb from dVazebTab where (typvazbyk = 'typVazbyDokl.odpocetZALOHY' or typvazbyk = 'typVazbyDokl.odpocetZDD')
                                                      and ((idtaba = i_idDokl and modula = i_modul and modulb = i_modul));
    if (_pocetVazeb <> 0) then
      return false;
    end if;
    
    return true;
  else
  
   /* Jedná se o doklad, kde je hlavička */
   select count(*) into _pocetVazeb from dVazebTab where (typvazbyk != 'typVazbyDokl.polozkaSkl') and (not (
                                                         (modula='FAV' and modulb='FAV' and typvazbyk='typVazbyDokl.odpocetZALOHY') or
                                                         (modula='FAV' and modulb='FAV' and typvazbyk='typVazbyDokl.odpocetZDD') or
                                                         (modula='FAP' and modulb='FAP' and typvazbyk='typVazbyDokl.odpocetZALOHY') or
                                                         (modula='FAP' and modulb='FAP' and typvazbyk='typVazbyDokl.odpocetZDD'))) and
                                                         ((idtaba = i_idDokl and modula = i_modul) or (idtabb = i_idDokl and modulb=i_modul));
   if (_pocetVazeb <> 0) then
     return false;
   end if;
     
   return true;
  end if;
end;
$$ LANGUAGE plpgsql;


-- oprava funkce vracejici koncovou cast nazvu tabulky dokladu/polozky pro dany modul (Fak/Int/Sklad...)
-- misto Obch vracelo Obchod
CREATE OR REPLACE FUNCTION getDoklNamePostfixByModul(modul varchar(20))
  RETURNS varchar(20) AS $$
declare
  _postfix varchar(20);
begin
  if (modul in( 'FAV','FAP','PHL','ZAV')) then
    _postfix := 'Fak';
  else
    if (modul in( 'SKL')) then
      _postfix := 'Sklad';
    else
      if (modul in( 'BAN','POK','INT')) then
        _postfix := 'Int';
      else
        if (modul in ('PPP', 'NAV', 'OBP', 'PPV', 'NAP', 'OBV')) then
          _postfix := 'Obch';
        else
          raise exception 'getDoklNamePostfixByModul(): Neznámý modul: %', modul;
        end if;
      end if;
    end if;
  end if;
  
  return _postfix;
end; 
$$ LANGUAGE plpgsql;


-- trigger mazajici vazby na dokladech
CREATE OR REPLACE FUNCTION ddokldeleteobchvazba()
  RETURNS trigger AS $$
declare
  _postfix varchar(20);
  _idDokl bigint;
begin
  select getDoklNamePostfixByModul(OLD.modul) into _postfix;
  if (_postfix = 'Fak') then
    _idDokl := OLD.idDoklFak;
  else
    if (_postfix = 'Obch') then
      _idDokl := OLD.idDoklObch;
    else
      if (_postfix = 'Sklad') then
        _idDokl := OLD.idDoklSklad;
      end if;
    end if;
  end if;
  
  delete from dVazebTab where (typvazbyk in ('typVazbyDokl.obchod_obchod_hla', 'typVazbyDokl.obchod_faktura_hla',
                                             'typVazbyDokl.obchod_zaloha_hla', 'typVazbyDokl.obchod_sklad_hla'))
                              and ((idTabA = _idDokl and modulA = OLD.modul) or
                                   (idTabB = _idDokl and modulB = OLD.modul));
  
  return OLD;
end;
$$ LANGUAGE plpgsql;
ALTER FUNCTION ddokldeleteobchvazba() OWNER TO %firmaRole%;


-- naveseni na tabulky dokladu
CREATE TRIGGER ddoklobchdeleteobchvazba BEFORE DELETE ON ddoklobch
  FOR EACH ROW EXECUTE PROCEDURE ddokldeleteobchvazba();
CREATE TRIGGER ddoklfakdeleteobchvazba BEFORE DELETE ON ddoklfak
  FOR EACH ROW EXECUTE PROCEDURE ddokldeleteobchvazba();
CREATE TRIGGER ddoklskladdeleteobchvazba BEFORE DELETE ON ddoklsklad
  FOR EACH ROW EXECUTE PROCEDURE ddokldeleteobchvazba();


-- trigger mazajici vazby na polozkach
CREATE OR REPLACE FUNCTION dpoldokldeleteobchvazba()
  RETURNS trigger AS $$
declare
  _postfix varchar(20);
  _idPolDokl bigint;
begin
  select getDoklNamePostfixByModul(OLD.modul) into _postfix;
  if (_postfix = 'Fak') then
    _idPolDokl := OLD.idPolFak;
  else
    if (_postfix = 'Obch') then
      _idPolDokl := OLD.idPolObch;
    else
      if (_postfix = 'Sklad') then
        _idPolDokl := OLD.idPolSklad;
      end if;
    end if;
  end if;
  
  delete from dVazebTab where (typvazbyk in ('typVazbyDokl.obchod_obchod_pol', 'typVazbyDokl.obchod_faktura_pol',
                                             'typVazbyDokl.obchod_zaloha_pol', 'typVazbyDokl.obchod_sklad_pol'))
                              and ((idTabA = _idPolDokl and modulA = OLD.modul) or
                                   (idTabB = _idPolDokl and modulB = OLD.modul));
  
  return OLD;
end;
$$ LANGUAGE plpgsql;
ALTER FUNCTION dpoldokldeleteobchvazba() OWNER TO %firmaRole%;


-- naveseni na tabulky polozek
CREATE TRIGGER dpolobchdeleteobchvazba BEFORE DELETE ON dpolobch
  FOR EACH ROW EXECUTE PROCEDURE dpoldokldeleteobchvazba();
CREATE TRIGGER dpolfakdeleteobchvazba BEFORE DELETE ON dpolfak
  FOR EACH ROW EXECUTE PROCEDURE dpoldokldeleteobchvazba();
CREATE TRIGGER dpolskladdeleteobchvazba BEFORE DELETE ON dpolsklad
  FOR EACH ROW EXECUTE PROCEDURE dpoldokldeleteobchvazba();


-- zvyseni cisla verze db
insert into dbversion(iddbversion, dbversion, needreimport)
        values (nextval('seq_dbversion'), 24, 't');