--------------------------------------------------------------------------------
-- Bug 18293 – [WinStrom] Zamykání období ve mzdách
--------------------------------------------------------------------------------

-- trigger odnastavujici datum vypoctu mzdy pri zmene a hlidajici ze se nezmeni slozka v zamcenem obdobi
CREATE OR REPLACE FUNCTION pmzdslozupdatedatumvypoctu()
  RETURNS trigger AS $$
declare
  ZAZNAM record;
  _datumOd date;
  _datumDo date;
  _zamku integer;
begin
  if (tg_op = 'DELETE') then
    ZAZNAM := OLD;  
  else  
    ZAZNAM := NEW;  
  end if;
  
  _datumOd = date (ZAZNAM.rok || '-' || ZAZNAM.mesic || '-01');
  _datumDo = _datumOd + interval '1 month';
  
  select count(*) into _zamku from wzamky where modulmzd = true and zamekk in ('zamek.polozamceno', 'zamek.zamceno') and
      platioddata < _datumDo and platidodata >= _datumOd;
  if (_zamku > 0) then
    raise exception 'Mzdy v období %/% jsou zamčené a není možno je měnit.', ZAZNAM.rok, ZAZNAM.mesic;
  end if;
  
  if (ZAZNAM.typslozkyk <> 'typSlozky.kopie' and ZAZNAM.skupslozkyk <> 'skupinaSlozky.doba') then
    update pmzdsloz set datumod = null where rok = ZAZNAM.rok and mesic = ZAZNAM.mesic and idosoba = ZAZNAM.idosoba and 
        typslozkyk = 'typSlozky.kopie' and skupslozkyk = 'skupinaSlozky.doba';
  end if;
  
  return ZAZNAM;
end;
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.pmzdslozupdatedatumvypoctu() OWNER TO %firmaRole%;

CREATE TRIGGER pmzdslozupdatedatumvypoctu AFTER INSERT OR UPDATE OR DELETE ON pmzdsloz
  FOR EACH ROW EXECUTE PROCEDURE pmzdslozupdatedatumvypoctu();

-- zvyseni verze databaze
insert into dbversion(iddbversion, dbversion, needreimport)
  values (nextval('seq_dbversion'), 261, 't');
