--------------------------------------------------------------------------------
-- Bug 22941 – [winstrom] 1 - Sumace dokladů v modulu prodej
--------------------------------------------------------------------------------

CREATE OR REPLACE FUNCTION dpoldoklupdatesumNaklady()
  RETURNS trigger AS
$BODY$
declare
  _sumNakladyRozdil numeric(15,2);
  _cenaMjNakupOLD numeric(19,6);
  _cenaMjNakupNEW numeric(19,6);
  _mnozMjOLD numeric(19,6);
  _mnozMjNEW numeric(19,6);
  _stornoOLD boolean;
  _stornoNEW boolean;
  _zaznamRETURN record;
begin
  if (tg_op = 'INSERT') then
    _cenaMjNakupOLD = 0.0;
    _cenaMjNakupNEW = NEW.cenaMjNakup;
    _mnozMjOLD = 0.0;
    _mnozMjNEW = NEW.mnozMj;
    _stornoOLD = false;
    _stornoNEW = NEW.storno;
    _zaznamRETURN = NEW;
  elsif(tg_op = 'UPDATE') then
    _cenaMjNakupOLD = OLD.cenaMjNakup;
    _cenaMjNakupNEW = NEW.cenaMjNakup;
    _mnozMjOLD = OLD.mnozMj;
    _mnozMjNEW = NEW.mnozMj;
    _stornoOLD = OLD.storno;
    _stornoNEW = NEW.storno;
    _zaznamRETURN = NEW;
  else
    _cenaMjNakupOLD = OLD.cenaMjNakup;
    _cenaMjNakupNEW = 0.0;
    _mnozMjOLD = OLD.mnozMj;
    _mnozMjNEW = 0.0;
    _stornoOLD = OLD.storno;
    _stornoNEW = false;
    _zaznamRETURN = OLD;
  end if;

  if(_cenaMjNakupOLD = _cenaMjNakupNEW and _mnozMjOLD = _mnozMjNEW and _stornoOLD = false and _stornoNEW = false) then 
    return _zaznamRETURN;
  end if;

  _sumNakladyRozdil = 0.0;
  if(_stornoOLD = false) then
    _sumNakladyRozdil = _sumNakladyRozdil - round(_mnozMjOLD * _cenaMjNakupOLD, 2);
  end if;

  if(_stornoNEW = false) then
    _sumNakladyRozdil = _sumNakladyRozdil + round(_mnozMjNEW * _cenaMjNakupNEW, 2);
  end if;

  if(_zaznamRETURN.modul = 'FAV') then
    update ddoklfak set
      sumNaklady = sumNaklady + _sumNakladyRozdil
      where idDoklFak = _zaznamRETURN.idDoklFak;
  elsif(_zaznamRETURN.modul = 'POK') then
    update ddoklint set
      sumNaklady = sumNaklady + _sumNakladyRozdil
      where idDoklInt = _zaznamRETURN.idDoklInt;
  elsif(_zaznamRETURN.modul = 'OBP' or _zaznamRETURN.modul = 'NAV' or _zaznamRETURN.modul = 'PPP') then
    update ddoklobch set
      sumNaklady = sumNaklady + _sumNakladyRozdil
      where idDoklobch = _zaznamRETURN.idDoklObch;
  end if;
  return _zaznamRETURN;
end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE
  COST 100;
ALTER FUNCTION dpoldoklupdatesumNaklady() OWNER TO %firmaRole%;


DROP TRIGGER dpolfakupdatesumNaklady ON dpolfak;
CREATE TRIGGER dpolfakupdatesumNaklady
  AFTER INSERT OR UPDATE OR DELETE
  ON dpolfak
  FOR EACH ROW
  EXECUTE PROCEDURE dpoldoklupdatesumNaklady();


DROP TRIGGER dpolintupdatesumNaklady ON dpolint;
CREATE TRIGGER dpolintupdatesumNaklady
  AFTER INSERT OR UPDATE OR DELETE
  ON dpolint
  FOR EACH ROW
  EXECUTE PROCEDURE dpoldoklupdatesumNaklady();


CREATE TRIGGER dpolobchupdatesumNaklady
  AFTER INSERT OR UPDATE OR DELETE
  ON dpolobch
  FOR EACH ROW
  EXECUTE PROCEDURE dpoldoklupdatesumNaklady();


-- naplneni nove pridaneho sloupecku
alter table ddoklfak disable trigger all;
UPDATE ddoklfak AS d
SET sumnaklady = pv.nakl
FROM (SELECT iddoklfak, sum(round(mnozmj * cenamjnakup, 2)) as nakl
      FROM dpolfak
      WHERE storno = false
      GROUP BY iddoklfak) pv
   WHERE d.iddoklfak = pv.iddoklfak;
alter table ddoklfak enable trigger all;

alter table ddoklint disable trigger all;
UPDATE ddoklint AS d
SET sumnaklady = pv.nakl
FROM (SELECT iddoklint, sum(round(mnozmj * cenamjnakup, 2)) as nakl
      FROM dpolint
      WHERE storno = false
      GROUP BY iddoklint) pv
   WHERE d.iddoklint = pv.iddoklint;
alter table ddoklint enable trigger all;

alter table ddoklobch disable trigger all;
UPDATE ddoklobch AS d
SET sumnaklady = pv.nakl
FROM (SELECT iddoklobch, sum(round(mnozmj * cenamjnakup, 2)) as nakl
      FROM dpolobch
      WHERE storno = false
      GROUP BY iddoklobch) pv
   WHERE d.iddoklobch = pv.iddoklobch;
alter table ddoklobch enable trigger all;


-- zvyseni verze databaze
insert into dbversion(iddbversion, dbversion, needreimport, appversion, upgradetime)
  values (nextval('seq_dbversion'), 457, 't', %appVersion%, clock_timestamp());
