-- Bug 12658 – [WinStrom] Tabulka pro daňovou evidenci a její správa

-- Sekvence pro tabulku pro daňovou evidenci
CREATE SEQUENCE seq_uDanEvid
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;
    
ALTER TABLE public.seq_uDanEvid OWNER TO %firmaRole%;

-- Tabulka pro daňovou evidenci
CREATE TABLE uDanEvid
   (idDanEvid integer DEFAULT nextval('seq_uDanEvid'::regclass) NOT NULL,
    danEvidK character varying(50),
    sumZkl numeric(15,2) DEFAULT 0.0 NOT NULL,
    sumDph numeric(15,2) DEFAULT 0.0 NOT NULL,
    sumZklMen numeric(15,2) DEFAULT 0.0 NOT NULL,
    sumDphMen numeric(15,2) DEFAULT 0.0 NOT NULL,
    kurz numeric(10,6) DEFAULT 0.0 NOT NULL,
    kurzMnozstvi integer DEFAULT 0 NOT NULL,
    idDoklInt integer NOT NULL,
    idDoklFak integer,
    idMeny integer,
    CONSTRAINT uDanEvid_pkey PRIMARY KEY (idDanEvid),
    CONSTRAINT uDanEvid_idDoklInt FOREIGN KEY (idDoklInt) REFERENCES ddoklint(iddoklint),
    CONSTRAINT uDanEvid_idDoklFak FOREIGN KEY (idDoklFak) REFERENCES ddoklfak(iddoklfak),
    CONSTRAINT uDanEvid_idMeny FOREIGN KEY (idmeny) REFERENCES umeny(idmeny)
  );
ALTER TABLE udanevid OWNER TO %firmaRole%;

CREATE INDEX uDanEvid_IdDoklInt_index ON uDanEvid USING btree (IdDoklInt);
CREATE INDEX uDanEvid_IdDoklFak_index ON uDanEvid USING btree (IdDoklFak);
CREATE INDEX uDanEvid_IdMeny_index ON uDanEvid USING btree (IdMeny);


-- Uložená procedura aktualizující tabulku daňové evidence pro interní doklad se zadaným id
create or replace function uUpdateDanEvid (in _idDoklInt integer) returns void as $$
declare
  doklInt record;
  tempRec record;
  podvoj boolean;
  navazaneDotaz varchar;
  vazeb integer;
  _danEvidK character varying(50);
  nespadaDph character varying(50);
  _idDanEvid integer;
  _idDoklFak integer;
  castka numeric(15,2);
  castkaMen numeric(15,2);
  castkaZbyva numeric(15,2);
  castkaZbyvaMen numeric(15,2);
  castkaOdpocet numeric(15,2);
  castkaOdpocetMen numeric(15,2);
  zapocteno numeric(15,2);
  zapoctenoMen numeric(15,2);
  domaciMena integer;
  ciziMena boolean;
begin
  nespadaDph := 'danEvid.NeDanEvid';

  select * into doklInt from ddoklint where iddoklint = _idDoklInt;
  select podvojucto, idmeny into podvoj, domaciMena from nnastav where (platioddata is null or platioddata < doklInt.datUcto) and (platidodata is null or platidodata > doklInt.datUcto);
  
  ciziMena := (doklInt.idMeny is not null and domaciMena <> doklInt.idMeny);
  
  if (doklInt.modul not in ('BAN', 'POK')) then
    return;
  end if;

  if (podvoj = false and doklInt.ucetni = true) then
    -- dotaz ktery nalezne veschny navazane faktury a navazane castky
    navazaneDotaz := '(select V.idDoklFak, sum(V.castka) as castka from
         (select idTabA as idDoklFak, castka, 1 as unionId from dvazebtab where typvazbyk = ''typVazbyDokl.uhrada'' and idtabb = ' || doklInt.idDoklInt || '
          union
          select P.idDoklFak, V.castka, 2 as unionId from dpolfak as P join dVazebTab as V on 
            (V.typvazbyk = ''typVazbyDokl.uhradaZALOHY'' and V.idTabB = ' || doklInt.idDoklInt || ' and V.idTabA in 
              (select idTabB from dVazebTab where typVazbyK = ''typVazbyDokl.odpocetZALOHY'' and idTabA = P.idPolFak))
          union
          select P.idDoklFak, V.castka, 3 as unionId from dpolfak as P join dVazebTab as V on 
            (V.typvazbyk = ''typVazbyDokl.vazbaZDD'' and V.idTabB = ' || doklInt.idDoklInt || ' and V.idTabA in 
              (select idTabB from dVazebTab where typVazbyK = ''typVazbyDokl.odpocetZDD'' and idTabA = P.idPolFak))
          ) as V group by V.idDoklFak)';
    
    execute 'select count(*) from ' || navazaneDotaz || ' as D' into vazeb;
  
    if (vazeb = 0) then			-- pro interni doklad neni vazba
      DELETE from uDanEvid where idDoklInt = doklInt.idDoklInt and (idDoklFak is not null or           -- vymazeme ty co byli vytvorene pres vazbu
            (danEvidK <> nespadaDph and                                                                -- a ty co nebudeme potrebovat krome nespada
             danEvidK not in (select danEvidK from dPolInt where idDoklInt = doklInt.idDoklInt and danEvidK is not null group by danEvidK)));
             
      -- DPH
      select idDanEvid into _idDanEvid from uDanEvid where danEvidK = nespadaDph and idDoklInt = doklInt.idDoklInt;  -- zjistime zda uz zaznam existuje
      select coalesce(sum(sumDph),0),coalesce(sum(sumDphMen),0) into castka,castkaMen from dPolInt 
        where idDoklInt = doklInt.idDoklInt and danEvidK is not null and storno = false;      -- secteme dane
      
      if (_idDanEvid is null) then              -- zaznam jeste nebyl vytvoren
        INSERT into uDanEvid (danEvidK, sumDph, sumDphMen, kurz, kurzMnozstvi, idDoklInt, idMeny) 
          VALUES (nespadaDph, castka, castkaMen, doklInt.kurz, doklInt.kurzmnozstvi, doklInt.idDoklInt, doklInt.idMeny);
      else                                     -- zaznam jiz byl vytvoren, vynulujeme i zaklad pokud by jiz nebyl pouzit
        UPDATE uDanEvid SET sumDph = castka, sumDphMen = castkaMen, sumZkl = 0, sumZklMen = 0, kurz = doklInt.kurz, 
          kurzMnozstvi = doklInt.kurzmnozstvi, idMeny = doklInt.idMeny, idDoklFak = null WHERE idDanEvid = _idDanEvid;
      end if;
    
      -- Zaklad
      for _danEvidK in select danEvidK from dPolInt where idDoklInt = doklInt.idDoklInt and danEvidK is not null group by danEvidK loop 
        select idDanEvid into _idDanEvid from uDanEvid where danEvidK = _danEvidK and idDoklInt = doklInt.idDoklInt;  -- zjistime zda uz zaznam existuje
        select sum(sumZkl),sum(sumZklMen) into castka,castkaMen from dPolInt where idDoklInt = doklInt.idDoklInt and danEvidK = _danEvidK and storno = false; -- secteme castky
          
        if (_idDanEvid is null) then              -- zaznam jeste nebyl vytvoren
          INSERT into uDanEvid (danEvidK, sumZkl, sumZklMen, kurz, kurzMnozstvi, idDoklInt, idMeny) 
            VALUES (_danEvidK, castka, castkaMen, doklInt.kurz, doklInt.kurzmnozstvi, doklInt.idDoklInt, doklInt.idMeny);
        else                                     -- zaznam jiz byl vytvoren
          UPDATE uDanEvid SET sumZkl = castka, sumZklMen = castkaMen, kurz = doklInt.kurz, kurzMnozstvi = doklInt.kurzmnozstvi,
            idMeny = doklInt.idMeny, idDoklFak = null WHERE idDanEvid = _idDanEvid;
        end if;
      end loop; 
    else				            -- pro interni doklad je vazba
      DELETE from uDanEvid where idDoklInt = doklInt.idDoklInt and idDoklFak is null;           -- smazeme polozky co nemaji vazbu na fakturu
      UPDATE uDanEvid set sumZkl = 0, sumDph = 0, sumZklMen = 0, sumDphMen = 0 where idDoklInt = doklInt.idDoklInt;     -- vynulujeme ostatni
      
      drop table if exists tempDanEvid;
      
      CREATE TEMPORARY TABLE tempDanEvid
        (idDoklFak integer NOT NULL,
         danEvidK character varying(50),
         kUhrade numeric(15,2) DEFAULT 0.0 NOT NULL,
         kUhradeMen numeric(15,2) DEFAULT 0.0 NOT NULL,
         uhrazeno numeric(15,2) DEFAULT 0.0 NOT NULL,
         uhrazenoMen numeric(15,2) DEFAULT 0.0 NOT NULL,
         vazbaFak numeric(15,2) DEFAULT 0.0 NOT NULL
        );

      for tempRec in execute 'select * from ' || navazaneDotaz || ' as D' loop
        _idDoklFak = tempRec.idDoklFak;
        castka = tempRec.castka;
      
        select * into tempRec from dDoklFak where idDoklFak = _idDoklFak;
        if ((ciziMena = true and (tempRec.idMeny is null or tempRec.idMeny <> doklInt.idMeny)) 
         or (ciziMena = false and tempRec.idMeny is not null and tempRec.idMeny <> domaciMena)) then
          raise exception 'uUpdateDanEvid: Nesouhlasi meny dokladu';
        end if;
      
        -- zjistime kolik je treba na fakture uhradit a kolik je uhrazeno
        insert into tempDanEvid (idDoklFak, danEvidK, vazbaFak, kUhrade, kUhradeMen, uhrazeno, uhrazenoMen) 
          select _idDoklFak, danEvidK, castka, sum(sumZkl), sum(sumZklMen), 
               (select coalesce(sum(sumZkl),0) from uDanEvid where idDoklFak = _idDoklFak and danEvidK = P.danEvidK),
               (select coalesce(sum(sumZklMen),0) from uDanEvid where idDoklFak = _idDoklFak and danEvidK = P.danEvidK)
           from dPolFak as P where idDoklFak = _idDoklFak and danEvidK is not null and storno = false
             and typPolozkyK not in ('typPolozky.odpocetZdd', 'typPolozky.odpocetZalohy') group by danEvidK;
           
        -- dan
        insert into tempDanEvid (idDoklFak, danEvidK, vazbaFak, kUhrade, kUhradeMen, uhrazeno, uhrazenoMen) values (_idDoklFak, null, castka,   -- dan si oznacime jako null
           (select coalesce(sum(sumDph),0) from dPolFak where idDoklFak = _idDoklFak and danEvidK is not null and storno = false 
              and typPolozkyK not in ('typPolozky.odpocetZdd', 'typPolozky.odpocetZalohy')),     -- castka dane dokladu
           (select coalesce(sum(sumDphMen),0) from dPolFak where idDoklFak = _idDoklFak and danEvidK is not null and storno = false
              and typPolozkyK not in ('typPolozky.odpocetZdd', 'typPolozky.odpocetZalohy')),  -- castka dane dokladu v mene
           (select coalesce(sum(sumDph),0) from uDanEvid where idDoklFak = _idDoklFak),                             -- uhrazena dan dokladu
           (select coalesce(sum(sumDphMen),0) from uDanEvid where idDoklFak = _idDoklFak)                           -- uhrazena dan dokladu v mene
          );
      end loop;
      
      zapocteno := 0;
      zapoctenoMen := 0;
      
      for _idDoklFak,castka in select idDoklFak,vazbaFak from tempDanEvid group by idDoklFak,vazbaFak loop           -- prochazime navazene faktury
        castkaMen := castka;

        for tempRec in select * from tempDanEvid where idDoklFak = _idDoklFak order by (case when (kUhrade-uhrazeno) < 0 then -1 else 1 end), -- napred zaporny 
                                                                                              danEvidK desc loop    -- sestupne aby null (dan) bylo prvni
        
          if ((ciziMena = false and castka > 0) or (ciziMena = true and castkaMen > 0)) then                -- pokud jeste mame nejakou castku
            castkaZbyva := tempRec.kUhrade - tempRec.uhrazeno;
            castkaZbyvaMen := tempRec.kUhradeMen - tempRec.uhrazenoMen;
            
            if ((ciziMena = false and castkaZbyva <> 0) or (ciziMena = true and castkaZbyvaMen <> 0)) then    -- pokud na aktualnim zaznamu zbyva neco uhradit
              castkaOdpocet := minValue(castka, castkaZbyva);      -- spocteme kolik budeme "odpocitavat"
              castkaOdpocetMen := minValue(castkaMen, castkaZbyvaMen);
              castka := castka - castkaOdpocet;                    -- odebereme to z odpocitatelne castky
              castkaMen := castkaMen - castkaOdpocetMen;
              zapocteno := zapocteno + castkaOdpocet;
              zapoctenoMen := zapoctenoMen + castkaOdpocetMen;
              
              if (ciziMena = true) then
                castkaOdpocet := castkaOdpocetMen * doklInt.kurz / doklInt.kurzmnozstvi;
              else
                castkaOdpocetMen := 0;
              end if;
              
              if (tempRec.danEvidK is null) then        -- pokud se jedna o dan
                select idDanEvid into _idDanEvid from uDanEvid where idDoklInt = doklInt.idDoklInt and idDoklFak = tempRec.idDoklFak and danEvidK = nespadaDph;  -- zjistime zda uz zaznam existuje
                
                if (_idDanEvid is null) then            -- zaznam jeste nebyl vytvoren
                  INSERT into uDanEvid (danEvidK, sumDph, sumDphMen, kurz, kurzMnozstvi, idDoklInt, idDoklFak, idMeny) 
                    VALUES (nespadaDph, castkaOdpocet, castkaOdpocetMen, doklInt.kurz, doklInt.kurzmnozstvi, doklInt.idDoklInt, tempRec.idDoklFak, doklInt.idMeny);
                else                                    -- zaznam jiz byl vytvoren
                  UPDATE uDanEvid SET sumDph = castkaOdpocet, sumDphMen = castkaOdpocetMen, 
                    kurz = doklInt.kurz, kurzMnozstvi = doklInt.kurzmnozstvi, idMeny = doklInt.idMeny WHERE idDanEvid = _idDanEvid;
                end if;
              else                                      -- pokud se nejedna o dan
                select idDanEvid into _idDanEvid from uDanEvid where idDoklInt = doklInt.idDoklInt and idDoklFak = tempRec.idDoklFak and danEvidK = tempRec.danEvidK;  -- zjistime zda uz zaznam existuje
                
                if (_idDanEvid is null) then            -- zaznam jeste nebyl vytvoren
                  INSERT into uDanEvid (danEvidK, sumZkl, sumZklMen, kurz, kurzMnozstvi, idDoklInt, idDoklFak, idMeny) 
                    VALUES (tempRec.danEvidK, castkaOdpocet, castkaOdpocetMen, doklInt.kurz, doklInt.kurzmnozstvi, doklInt.idDoklInt, tempRec.idDoklFak, doklInt.idMeny);
                else                                    -- zaznam jiz byl vytvoren
                  UPDATE uDanEvid SET sumZkl = castkaOdpocet, sumZklMen = castkaOdpocetMen, 
                    kurz = doklInt.kurz, kurzMnozstvi = doklInt.kurzmnozstvi, idMeny = doklInt.idMeny WHERE idDanEvid = _idDanEvid;
                end if;
              end if;
            
              select idDanEvid into _idDanEvid from uDanEvid where idDoklInt = doklInt.idDoklInt and idDoklFak = tempRec.idDoklFak and
                 ((tempRec.danEvidK is not null and danEvidK = tempRec.danEvidK) or (tempRec.danEvidK is null and danEvidK = nespadaDph));  -- zjistime zda uz zaznam existuje
            end if;
          end if;
        
        end loop;
      end loop;
      
      if ((ciziMena = false and zapocteno < doklInt.sumCelkem) or (ciziMena = true and zapoctenoMen < doklInt.sumCelkemMen)) then     -- pokud neni komplet sparovan
        castka = doklInt.sumCelkem - zapocteno;
        castkaMen = doklInt.sumCelkemMen - zapoctenoMen;
        if (ciziMena = true) then
          castka := castkaMen * doklInt.kurz / doklInt.kurzmnozstvi;
        else
          castkaMen := 0;
        end if;
        
        INSERT into uDanEvid (danEvidK, sumZkl, sumZklMen, kurz, kurzMnozstvi, idDoklInt, idMeny) 
          VALUES ((select danEvidK from uDanEvid where idDoklInt = doklInt.idDoklInt order by sumZkl desc limit 1),     -- vybereme typ pro ktery je nejvetsi castka
           castka, castkaMen, doklInt.kurz, doklInt.kurzmnozstvi, doklInt.idDoklInt, doklInt.idMeny);
      end if;
      
    end if;
  end if;
  
  DELETE from uDanEvid WHERE idDoklInt = doklInt.idDoklInt and sumZkl = 0 and sumZklMen = 0 and sumDph = 0 and sumDphMen = 0;
end;
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.uUpdateDanEvid(integer) OWNER TO %firmaRole%;


-- trigger volaný při vytvoření a aktualizaci položky interního dokladu - aktualizuje položky v daňové evidenci
CREATE OR REPLACE FUNCTION dpolintupdatedanevid()
  RETURNS trigger AS $$
declare
  ZAZNAM record;
begin
  if (tg_op = 'DELETE') then  
    ZAZNAM := OLD;  
  else  
    ZAZNAM := NEW;  
  end if;

  perform uUpdateDanEvid(ZAZNAM.idDoklInt);
  
  return ZAZNAM;
end;
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.dpolintupdatedanevid() OWNER TO %firmaRole%;

-- navěšení triggeru na tabulku položek interních dokladů
CREATE TRIGGER dpolintupdatedanevid AFTER INSERT OR UPDATE OR DELETE ON dpolint
  FOR EACH ROW EXECUTE PROCEDURE dpolintupdatedanevid();
  
  
-- trigger volaný při změně položky faktury - aktualizuje položky v daňové evidenci
CREATE OR REPLACE FUNCTION dpolfakupdatedanevid()
  RETURNS trigger AS $$
declare
  idDoklInt integer;
begin
  for idDoklInt in (
    select idTabB from dVazebTab where idTabA = NEW.idDoklFak and typVazbyK = 'typVazbyDokl.uhrada'
    union
    select idTabB from dVazebTab where typVazbyK = 'typVazbyDokl.uhradaZALOHY' and idTabA in
      (select idTabB from dVazebTab where typVazbyK = 'typVazbyDokl.odpocetZALOHY' and idTabA in
        (select idPolFak from dPolFak where idDoklFak = NEW.idDoklFak))
    union
    select idTabB from dVazebTab where typVazbyK = 'typVazbyDokl.vazbaZDD' and idTabA in
      (select idTabB from dVazebTab where typVazbyK = 'typVazbyDokl.odpocetZDD' and idTabA in
        (select idPolFak from dPolFak where idDoklFak = NEW.idDoklFak))
  ) loop
    perform uUpdateDanEvid(idDoklInt);
  end loop;
  
  return NEW;
end;
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.dpolfakupdatedanevid() OWNER TO %firmaRole%;

-- navěšení triggeru na tabulku položek interních dokladů
CREATE TRIGGER dpolfakupdatedanevid AFTER UPDATE ON dpolfak
  FOR EACH ROW EXECUTE PROCEDURE dpolfakupdatedanevid();
  
  
-- trigger volaný při vytváření a mazání vazby mezi doklady - aktualizuje položky v daňové evidenci pokud se jedná o úhradu
CREATE OR REPLACE FUNCTION dvazebtabinsertdeletedanevid()
  RETURNS trigger AS $$
declare
  ZAZNAM record;
  _idDoklInt integer;
begin
  if (tg_op = 'DELETE') then  
    ZAZNAM := OLD;  
  else  
    ZAZNAM := NEW;  
  end if;
  _idDoklInt := null;
  
  if (ZAZNAM.typvazbyk in ('typVazbyDokl.uhrada', 'typVazbyDokl.uhradaZALOHY')) then
    _idDoklInt := ZAZNAM.idTabB;
  end if;
  if (ZAZNAM.typvazbyk = 'typVazbyDokl.odpocetZALOHY') then
    select idTabB into _idDoklInt from dVazebTab where typVazbyK = 'typVazbyDokl.uhradaZALOHY' and idTabA = ZAZNAM.idTabB;
  end if;
  if (ZAZNAM.typvazbyk = 'typVazbyDokl.odpocetZDD') then
    select idTabB into _idDoklInt from dVazebTab where typVazbyK = 'typVazbyDokl.vazbaZDD' and idTabA = ZAZNAM.idTabB;
  end if;
  
  if (_idDoklInt is not null) then
    perform uUpdateDanEvid(_idDoklInt);
  end if;
  
  return ZAZNAM;
end;
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.dvazebtabinsertdeletedanevid() OWNER TO %firmaRole%;

-- navěšení triggeru na tabulku vazeb
CREATE TRIGGER dvazebtabinsertdeletedanevid AFTER INSERT OR DELETE ON dvazebtab
  FOR EACH ROW EXECUTE PROCEDURE dvazebtabinsertdeletedanevid();


-- zvyseni verze databaze
insert into dbversion(iddbversion, dbversion, needreimport)
  values (nextval('seq_dbversion'), 48, 't');
