--------------------------------------------------------------------------------
-- Bug 14296 – [WinStrom] Průvodce nastavením počátečních stavů účtů
--------------------------------------------------------------------------------

ALTER TABLE uobraty
    ADD COLUMN obrdalbck numeric(15,2),
    ADD COLUMN obrmdbck numeric(15,2),
    ADD COLUMN uzivnastav boolean DEFAULT false NOT NULL;

ALTER TABLE uobraty DISABLE TRIGGER ALL;

UPDATE uobraty
SET obrdalbck=obrdal, obrmdbck=obrmd
WHERE obdobi=0;

ALTER TABLE uobraty ENABLE TRIGGER ALL;



CREATE OR REPLACE FUNCTION uuzavriucetniobdobi(idoldobdobi integer, idnewobdobi integer, iducet701 integer, iducet702 integer, iducet710 integer, iducet431 integer)
  RETURNS void AS
$BODY$
declare
  newRok integer;
  newObrat record;
  pom integer;
begin
  select date_part('year', platiOdData) into newRok from nUcetObdobi where idUcetObdobi = idNewObdobi;
  
  /*smazani pripadne predchozi zaverky*/
  delete from uObraty where idUcetObdobi = idOldObdobi and obdobi = 99;
  
  /* [1] Sesumovani obratu MD/D a zauctovani uzavreni vsech uctu v obdobi 99 */
  /* Protistranou techto ucetnich operaci bude pro rozvahove ucty ucet 702 (viz bod [3])
     a pro vysledkove ucty ucet 710 (viz bod [4]) */
  insert into uObraty( idMeny,idUcet,idStred,idCinnost,Rok,Obdobi,ObrDal,ObrMd,idUcetObdobi,LastUpdate) (
    select O.idMeny,O.idUcet,O.idStred,O.idCinnost,min(O.Rok) as Rok,99 as Obdobi,sum(O.ObrMd) as ObrDal,sum(O.ObrDal) as ObrMd,O.idUcetObdobi,now() as LastUpdate
    from uObraty as O left outer join uUcty as U on U.idUcet = O.idUcet
    where O.idUcetObdobi = idOldObdobi and U.typUctuK <> 'typUctu.zaverkovy'
    group by O.idMeny,O.idUcet,O.idStred,O.idCinnost,O.idUcetObdobi);

  /* [2] Presun zustatku rozvahovych uctu do obdobi 0 noveho roku */
  /* smazani nepotrebnych pocatecnich zustatku pro bod [2] */
  delete from uObraty o1
  where o1.idUcetObdobi = idNewObdobi and o1.obdobi = 0 and o1.uzivnastav = false
  and o1.idUcet <> idUcet701
  and not exists (select 1 from uObraty o2 
                  where o2.idUcetObdobi = idoldobdobi and o2.obdobi = 99
                    and o2.idMeny = o1.idMeny
                    and o2.idUcet = o1.idUcet and o2.idStred = o1.idStred and compareid(o2.idCinnost, o1.idCinnost));

  /* vynulovani pocatecnich zustatku pro bod [2], [5] a *[6]* */
  update uObraty
  set obrdal = 0, obrmd = 0, obrdalbck = 0, obrmdbck = 0
  where idUcetObdobi = idNewObdobi and obdobi = 0 and uzivnastav = false;

  update uObraty
  set obrdalbck = 0, obrmdbck = 0
  where idUcetObdobi = idNewObdobi and obdobi = 0 and uzivnastav = true
    and obrdalbck is not null and obrmdbck is not null;

  /* aktualizace pocatecnich zustatku */
  for newObrat in (select O.idMeny,O.idUcet,O.idStred,O.idCinnost,newRok as Rok,0 as Obdobi,
                     sum(O.ObrMd) as ObrDal,sum(O.ObrDal) as ObrMd, idNewObdobi as idUcetObdobi, now() as LastUpdate
                   from uObraty as O left outer join uUcty as U on U.idUcet = O.idUcet
                   where O.idUcetObdobi = idOldObdobi and O.obdobi = 99 and U.typUctuK in( 'typUctu.rozvahovy','typUctu.podrozvahovy','typUctu.vnitropodnikovy') 
                   group by O.idMeny,O.idUcet,O.idStred,O.idCinnost,O.idUcetObdobi,U.typUctuK) loop
        select count(1) into pom from uObraty
        where obdobi = newObrat.Obdobi and rok = newObrat.Rok and idmeny = newObrat.idMeny
          and iducet = newObrat.idUcet and idstred = newObrat.idStred and compareid(idCinnost, newObrat.idCinnost)
          and iducetobdobi = newObrat.idUcetObdobi;
        if (pom > 0) then
            update uObraty 
            set obrdal = newObrat.ObrDal, obrmd = newObrat.ObrMd,
                obrdalbck = newObrat.ObrDal, obrmdbck = newObrat.ObrMd
            where obdobi = newObrat.Obdobi and rok = newObrat.Rok and idmeny = newObrat.idMeny
              and iducet = newObrat.idUcet and idstred = newObrat.idStred and compareid(idCinnost, newObrat.idCinnost)
              and iducetobdobi = newObrat.idUcetObdobi
              and uzivnastav = false;
           
            update uObraty 
            set obrdalbck = newObrat.ObrDal, obrmdbck = newObrat.ObrMd
            where obdobi = newObrat.Obdobi and rok = newObrat.Rok and idmeny = newObrat.idMeny
              and iducet = newObrat.idUcet and idstred = newObrat.idStred and compareid(idCinnost, newObrat.idCinnost)
              and iducetobdobi = newObrat.idUcetObdobi
              and uzivnastav = true;
        else
            insert into uObraty(idMeny,idUcet,idStred,idCinnost,Rok,Obdobi,
                                ObrDal, ObrMd,ObrDalBck,ObrMdBck,idUcetObdobi,LastUpdate)
            values (newObrat.idMeny,newObrat.idUcet,newObrat.idStred,newObrat.idCinnost,newObrat.Rok,newObrat.Obdobi,
                    newObrat.ObrDal,newObrat.ObrMd,newObrat.ObrDal,newObrat.ObrMd,newObrat.idUcetObdobi,newObrat.LastUpdate);
        end if;
  end loop;

  /* [3] Sumace na zaverkovy ucet 702 - protistrana k ucetnim operacim z bodu [1a] */
  insert into uObraty( idMeny,idUcet,idStred,idCinnost,Rok,Obdobi,ObrDal,ObrMd,idUcetObdobi,LastUpdate) (
    select O.idMeny,idUcet702 as idUcet,O.idStred,O.idCinnost,min(Rok) as Rok,99 as Obdobi,sum(O.ObrMd) as ObrDal,sum(O.ObrDal) as ObrMd,O.idUcetObdobi,now() as LastUpdate
    from uObraty as O left outer join uUcty as U on U.idUcet = O.idUcet
    where O.idUcetObdobi = idOldObdobi and O.obdobi = 99 and U.typUctuK in( 'typUctu.rozvahovy','typUctu.podrozvahovy','typUctu.vnitropodnikovy') 
    group by O.idMeny,O.idStred,O.idCinnost,O.idUcetObdobi);

  /* [4] Sumace na zaverkovy ucet 710 - protistrana k ucetnim operacim z bodu [1b] */
  insert into uObraty( idMeny,idUcet,idStred,idCinnost,Rok,Obdobi,ObrDal,ObrMd,idUcetObdobi,LastUpdate) (
    select O.idMeny,idUcet710 as idUcet,O.idStred,O.idCinnost,min(Rok) as Rok,99 as Obdobi,sum(O.ObrMd) as ObrDal,sum(O.ObrDal) as ObrMd,O.idUcetObdobi,now() as LastUpdate
    from uObraty as O left outer join uUcty as U on U.idUcet = O.idUcet
    where O.idUcetObdobi = idOldObdobi and O.obdobi = 99 and U.typUctuK = 'typUctu.vysledkovy'
    group by O.idMeny,O.idStred,O.idCinnost,O.idUcetObdobi);

  /* [5] Presun uctu 702 na ucet 701 v novem roce (s prohozenim stran) */
  for newObrat in (select O.idMeny,idUcet701 as idUcet,O.idStred,O.idCinnost,newRok as Rok,0 as Obdobi,
                     sum(O.ObrMd) as ObrDal,sum(O.ObrDal) as ObrMd, idNewObdobi as idUcetObdobi, now() as LastUpdate
                   from uObraty as O
                   where O.idUcetObdobi = idOldObdobi and O.obdobi = 99 and O.idUcet = idUcet702
                   group by O.idMeny,O.idStred,O.idCinnost,O.idUcetObdobi) loop
        select count(1) into pom from uObraty
        where obdobi = newObrat.Obdobi and rok = newObrat.Rok and idmeny = newObrat.idMeny
          and iducet = newObrat.idUcet and idstred = newObrat.idStred and compareid(idCinnost, newObrat.idCinnost)
          and iducetobdobi = newObrat.idUcetObdobi;
        if (pom > 0) then
            update uObraty 
            set obrdal = newObrat.ObrDal, obrmd = newObrat.ObrMd,
                obrdalbck = newObrat.ObrDal, obrmdbck = newObrat.ObrMd
            where obdobi = newObrat.Obdobi and rok = newObrat.Rok and idmeny = newObrat.idMeny
              and iducet = newObrat.idUcet and idstred = newObrat.idStred and compareid(idCinnost, newObrat.idCinnost)
              and iducetobdobi = newObrat.idUcetObdobi
              and uzivnastav = false;
           
            update uObraty 
            set obrdalbck = newObrat.ObrDal, obrmdbck = newObrat.ObrMd
            where obdobi = newObrat.Obdobi and rok = newObrat.Rok and idmeny = newObrat.idMeny
              and iducet = newObrat.idUcet and idstred = newObrat.idStred and compareid(idCinnost, newObrat.idCinnost)
              and iducetobdobi = newObrat.idUcetObdobi
              and uzivnastav = true;
        else
            insert into uObraty(idMeny,idUcet,idStred,idCinnost,Rok,Obdobi,
                                ObrDal, ObrMd,ObrDalBck,ObrMdBck,idUcetObdobi,LastUpdate)
            values (newObrat.idMeny,newObrat.idUcet,newObrat.idStred,newObrat.idCinnost,newObrat.Rok,newObrat.Obdobi,
                    newObrat.ObrDal,newObrat.ObrMd,newObrat.ObrDal,newObrat.ObrMd,newObrat.idUcetObdobi,newObrat.LastUpdate);
        end if;
  end loop;
      
  /* [6] Presun zustatku uctu 710 ze stareho roku na ucet 701 v novem roce (s prohozenim stran) */
  for newObrat in (select idMeny,idUcet701 as idUcet,idStred,idCinnost,newRok as Rok,0 as Obdobi,
                     sum(O.ObrMd) as ObrDal,sum(O.ObrDal) as ObrMd, idNewObdobi as idUcetObdobi, now() as LastUpdate
                   from uObraty as O
                   where O.idUcetObdobi = idOldObdobi and O.obdobi = 99 and O.idUcet = idUcet710
                   group by O.idMeny,O.idStred,O.idCinnost,O.idUcetObdobi) loop
	select count(1) into pom from uObraty
		where obdobi = newObrat.Obdobi and rok = newObrat.Rok and idmeny = newObrat.idMeny
		and iducet = newObrat.idUcet and idstred = newObrat.idStred and compareid(idCinnost, newObrat.idCinnost)
		and iducetobdobi = newObrat.idUcetObdobi;
	if (pom > 0) then
            update uObraty 
            set obrdal = obrdal + newObrat.ObrDal, obrmd = obrmd + newObrat.ObrMd,
                obrdalbck = obrdalbck + newObrat.ObrDal, obrmdbck = obrmdbck + newObrat.ObrMd
            where obdobi = newObrat.Obdobi and rok = newObrat.Rok and idmeny = newObrat.idMeny
              and iducet = newObrat.idUcet and idstred = newObrat.idStred and compareid(idCinnost, newObrat.idCinnost)
              and iducetobdobi = newObrat.idUcetObdobi
              and uzivnastav = false;
           
            update uObraty 
            set obrdalbck = obrdalbck + newObrat.ObrDal, obrmdbck = obrmdbck + newObrat.ObrMd
            where obdobi = newObrat.Obdobi and rok = newObrat.Rok and idmeny = newObrat.idMeny
              and iducet = newObrat.idUcet and idstred = newObrat.idStred and compareid(idCinnost, newObrat.idCinnost)
              and iducetobdobi = newObrat.idUcetObdobi
              and uzivnastav = true;
	else
            insert into uObraty(idMeny,idUcet,idStred,idCinnost,Rok,Obdobi,
                                ObrDal, ObrMd,ObrDalBck,ObrMdBck,idUcetObdobi,LastUpdate)
            values (newObrat.idMeny,newObrat.idUcet,newObrat.idStred,newObrat.idCinnost,newObrat.Rok,newObrat.Obdobi,
                    newObrat.ObrDal,newObrat.ObrMd,newObrat.ObrDal,newObrat.ObrMd,newObrat.idUcetObdobi,newObrat.LastUpdate);
	end if;
  end loop;

  /* [7] Presun zustatku uctu 710 ze stareho roku na ucet 431 v novem roce (bez prohozeni stran) */
  for newObrat in (select idMeny,idUcet431 as idUcet,idStred,idCinnost,newRok as Rok,0 as Obdobi,
                   sum(O.ObrDal) as ObrDal,sum(O.ObrMd) as ObrMd,idNewObdobi as idUcetObdobi, now() as LastUpdate
                   from uObraty as O
                   where O.idUcetObdobi = idOldObdobi and O.obdobi = 99 and O.idUcet = idUcet710
                   group by O.idMeny,O.idStred,O.idCinnost,O.idUcetObdobi) loop
	/** idUcet je 431 a ten jiz mohl byt zalozen, pokud na nej jiz uctovali */
	select count(1) into pom from uObraty
		where obdobi = newObrat.Obdobi and rok = newObrat.Rok and idmeny = newObrat.idMeny
		and iducet = newObrat.idUcet and idstred = newObrat.idStred and compareid(idCinnost, newObrat.idCinnost)
		and iducetobdobi = newObrat.idUcetObdobi;
	if (pom > 0) then
            update uObraty 
            set obrdal = obrdal + newObrat.ObrDal, obrmd = obrmd + newObrat.ObrMd,
                obrdalbck = obrdalbck + newObrat.ObrDal, obrmdbck = obrmdbck + newObrat.ObrMd
            where obdobi = newObrat.Obdobi and rok = newObrat.Rok and idmeny = newObrat.idMeny
              and iducet = newObrat.idUcet and idstred = newObrat.idStred and compareid(idCinnost, newObrat.idCinnost)
              and iducetobdobi = newObrat.idUcetObdobi
              and uzivnastav = false;
           
            update uObraty 
            set obrdalbck = obrdalbck + newObrat.ObrDal, obrmdbck = obrmdbck + newObrat.ObrMd
            where obdobi = newObrat.Obdobi and rok = newObrat.Rok and idmeny = newObrat.idMeny
              and iducet = newObrat.idUcet and idstred = newObrat.idStred and compareid(idCinnost, newObrat.idCinnost)
              and iducetobdobi = newObrat.idUcetObdobi
              and uzivnastav = true;
	else
            insert into uObraty(idMeny,idUcet,idStred,idCinnost,Rok,Obdobi,
                                ObrDal, ObrMd,ObrDalBck,ObrMdBck,idUcetObdobi,LastUpdate)
            values (newObrat.idMeny,newObrat.idUcet,newObrat.idStred,newObrat.idCinnost,newObrat.Rok,newObrat.Obdobi,
                    newObrat.ObrDal,newObrat.ObrMd,newObrat.ObrDal,newObrat.ObrMd,newObrat.idUcetObdobi,newObrat.LastUpdate);
	end if;
  end loop;

end;
$BODY$
  LANGUAGE 'plpgsql';

ALTER FUNCTION uuzavriucetniobdobi(idoldobdobi integer, idnewobdobi integer, iducet701 integer, iducet702 integer, iducet710 integer, iducet431 integer) OWNER TO %firmaRole%;



-- zvyseni verze databaze
insert into dbversion(iddbversion, dbversion, needreimport, appversion, upgradetime)
  values (nextval('seq_dbversion'), 590, 't', %appVersion%, clock_timestamp());
