------------------------------------------------------------------------------------------
-- Bug 30894 – [WinStrom] - Požadavky na výdej - vypočítat skutečný (záporný) stav skladu
------------------------------------------------------------------------------------------

alter table skarty add column stavMjSPozadavky numeric(19,6) NOT NULL DEFAULT 0.0;
alter table skarty add column pozadavkyMj numeric(19,6) NOT NULL DEFAULT 0.0;

-- vyplneni pozadavku a stavu s pozadavky na skladovych kartach
update skarty as s
set pozadavkyMj = pozadavkyMj + (ps.mnozMjPlan)
from dpolsklad as ps
where ps.idkarty = s.idkarty and ps.mnozMj = 0.0 and ps.mnozMjPlan != 0.0;

update skarty s set stavMjSPozadavky = stavMj - pozadavkyMj where pozadavkyMj != 0.0;

-- Function: daapolskladinsertmnozmj()
-- DROP FUNCTION daapolskladinsertmnozmj();
CREATE OR REPLACE FUNCTION daapolskladinsertmnozmj()
  RETURNS trigger AS
$BODY$  
declare  
  _stavSkladu numeric(19,6);
  _pozadavkyMj numeric(19,6);
  _stavMjSPozadavky numeric(19,6);
  _typPohybuK character varying; 
  _sklGenerovatPozadavky boolean; 
begin 
  -- tento trigger tu musi zustat protoze jinak na obycejnych vydejkach a prijemkach nebude vyplneno mnozMjPlan 
  select sklgenerovatpozadavky into _sklGenerovatPozadavky from nNastav where NEW.datVyst >= PlatiOdData order by PlatiOdData desc limit 1;  
  if (_sklGenerovatPozadavky is null) then  
        select sklgenerovatpozadavky into _sklGenerovatPozadavky from nNastav where PlatiOdData is null limit 1;   
  end if;  
 
  NEW.mnozMjPlan = NEW.mnozMj; 
 
  if (_sklGenerovatPozadavky) then 
    select typPohybuK into _typPohybuK from ddoklsklad where iddoklsklad = NEW.idDoklSklad; 
    select stavMj, pozadavkyMj, stavMjSPozadavky into _stavSkladu, _pozadavkyMj, _stavMjSPozadavky from skarty where idKarty = NEW.idKarty; 
   
    if(_typPohybuK = 'typPohybu.vydej' and NEW.mnozMj > _stavSkladu) then 
      update skarty set
          pozadavkyMj = _pozadavkyMj + NEW.mnozMj,
          stavMjSPozadavky = _stavMjSPozadavky - NEW.mnozMj
        where idkarty = NEW.idKarty;
      NEW.mnozMj = 0.0; 
    end if; 
  end if; 
   
  return NEW; 
end;  
$BODY$
  LANGUAGE plpgsql VOLATILE
  COST 100;
ALTER FUNCTION daapolskladinsertmnozmj() OWNER TO %firmaRole%;

-- Function: skladprepoctirozsirenoupolozku(integer, integer)
-- DROP FUNCTION skladprepoctirozsirenoupolozku(integer, integer);
CREATE OR REPLACE FUNCTION skladprepoctirozsirenoupolozku(i_idrozsirenepolozky integer, sign integer)
  RETURNS void AS
$BODY$ 
declare 
  _skladKarta record; 
  _rozsirenaPolozka record; 
  _polSklad record; 
  _navazanaPolSklad record; 
  _stavMj numeric(19,6); 
  _stavTuz numeric(19,6); 
  _stavMen numeric(19,6); 
  _prumCenaTuz numeric(19,6); 
  _prumCenaMen numeric(19,6); 
  _pocetNavazanych integer; 
  _cenaMj numeric(19,6); 
  _sumZkl numeric(19,6); 
  _sumZklMen numeric(19,6); 
  _kurzDokladu numeric(19,6); 
  _kurzSkladKarty numeric(19,6); 
  _isCiziMena boolean; 
  _sklfifo boolean; 
  _datPosl date;  
  _PoslCenaTuz numeric(19,6);      
  _PoslCenaMen numeric(19,6);      
  _cenaMjSkl numeric(19,6); 
  _updateCena boolean; 
  _rozpadlaPolSklad record; 
  _sumZklProCenaMj numeric(19,6); 
  _mnozMjProCenaMj numeric(19,6); 
  _cenaMjPoriz numeric(19,6); 
  --_cenaMjNakl numeric(19,6); 
begin 
  select * into _rozsirenaPolozka from tmp_RozsirenePolozky where idpolsklad = i_idrozsirenepolozky; 
  select * into _polSklad from dPolSklad where idPolSklad = i_idrozsirenepolozky; 
  if(_polSklad is null) then 
    -- muze se stat ze v rozsirenych polozkach je uz polozka ktera byla v ramci transakce smazana 
    return; 
  end if; 
   
  select * into _skladKarta from sKarty where idKarty = _rozsirenaPolozka.idskladkarty; 
  select getKurzFromDoklad(_rozsirenaPolozka.idDoklSkladPolozky, _polSklad.datVyst) into _kurzDokladu; 
 
  if(sign = 1) then 
    if(_rozsirenaPolozka.skupina = 1) then 
      if(_polSklad.mnozMj = 0.0) then 
        _stavTuz = _polSklad.cenaMj * _kurzDokladu; 
      else 
        select sumZkl into _stavTuz from dpolsklad where idPolSklad = _rozsirenaPolozka.idPolSklad; 
      end if; 
    else 
      -- pokud se jedna o prijemku z prevodky nebo vyroby tak se musi cena nastavit podle navazane vydejky 
      _cenaMjSkl = 0.0; 
      _sklfifo := (select isfifosklad(_polSklad.datVyst)); 
      -- menit cenu budeme jen pokud to neni fifo 
      _updateCena = not _sklFifo; 
      if(_rozsirenaPolozka.skupina = 2 and _rozsirenaPolozka.typPohybuK = 'typPohybu.prijem') then 
        select * into _navazanaPolSklad from dPolSklad  
            where idPolSklad = (select idTabA from dvazebtab where idTabB = _rozsirenaPolozka.idPolSklad and modulB = 'SKL' and typVazbyK = 'typVazbyDokl.polozkaSklPrevod'); 
        if(_navazanaPolSklad is null) then 
          _cenaMj = _polSklad.cenaMjNeskl; 
          _cenaMjSkl = 0.0; 
          _cenaMjPoriz = _cenaMj; 
          _sumZkl = _polSklad.cenaMjNeskl * _polSklad.mnozMj; 
          for _navazanaPolSklad in (select * from dPolSklad where idPolSklad in  
            (select idTabB from dvazebtab where idTabA = _rozsirenaPolozka.idPolSklad and modulA = 'SKL' and typVazbyK = 'typVazbyDokl.polozkaSklKus') ) 
            loop 
            if(_navazanaPolSklad.mnozMj <> 0.0) then 
              _cenaMj = _cenaMj + (_navazanaPolSklad.sumZkl / _polSklad.mnozMj) / _kurzDokladu; 
              _cenaMjSkl = _cenaMjSkl + (_navazanaPolSklad.sumZkl / _polSklad.mnozMj) / _kurzDokladu; 
            else 
              raise exception 'mnozMj je 0.0'; -- TODO vyresit co se bude dit 
            end if; 
            _sumZkl = _sumZkl + _navazanaPolSklad.sumZkl; 
 
            -- musime zohlednit i pripadne rozpady skladovych polozek 
            for _rozpadlaPolSklad in (select * from dPolSklad where idPolSklad in 
              (select idpolskladcil from svazbypolsklad where idpolskladzdroj = _navazanaPolSklad.idPolSklad)) 
            loop 
              if(_rozpadlaPolSklad.mnozMj <> 0.0) then 
                _cenaMj = _cenaMj + (_rozpadlaPolSklad.sumZkl / _polSklad.mnozMj) / _kurzDokladu; 
                _cenaMjSkl = _cenaMjSkl + (_rozpadlaPolSklad.sumZkl / _polSklad.mnozMj) / _kurzDokladu; 
              else 
                raise exception 'mnozMj je 0.0'; -- TODO vyresit co se bude dit 
              end if; 
              _sumZkl = _sumZkl + _rozpadlaPolSklad.sumZkl; 
            end loop; 
 
            _cenaMjPoriz = _cenaMj; 
 
            -- ovsem pokud se jedna o vyrobu musime zmenit cenu prijmky z vyroby 
            _updateCena = true; 
          end loop; 
          _cenaMj = _cenaMjPoriz + _polSklad.cenaMjNakl; 
          _sumZkl = _sumZkl + (_polSklad.mnozMj * _polSklad.cenaMjNakl); 
        else 
          _sumZklProCenaMj = 0.0; 
          _mnozMjProCenaMj = 0.0; 
          if(_navazanaPolSklad.mnozMj <> 0.0) then 
            _sumZklProCenaMj = _sumZklProCenaMj + _navazanaPolSklad.sumZkl / _kurzDokladu; 
            _mnozMjProCenaMj = _mnozMjProCenaMj + _navazanaPolSklad.mnozMj; 
          else 
            raise exception 'mnozMj je 0.0'; -- TODO vyresit co se bude dit 
          end if; 
          _sumZkl = _navazanaPolSklad.sumZkl; 
          -- musime zohlednit i pripadne rozpady skladovych polozek 
          for _rozpadlaPolSklad in (select * from dPolSklad where idPolSklad in 
            (select idpolskladcil from svazbypolsklad where idpolskladzdroj = _navazanaPolSklad.idPolSklad order by idpolskladcil desc) 
            order by idpolsklad desc)  
          loop 
            if(_polSklad.mnozMj > _mnozMjProCenaMj) then 
              if(_rozpadlaPolSklad.mnozMj <> 0.0) then 
                _sumZklProCenaMj = _sumZklProCenaMj + _rozpadlaPolSklad.sumZkl / _kurzDokladu; 
                _mnozMjProCenaMj = _mnozMjProCenaMj + _rozpadlaPolSklad.mnozMj; 
              else 
                raise exception 'mnozMj je 0.0'; -- TODO vyresit co se bude dit 
              end if; 
              _sumZkl = _sumZkl + _rozpadlaPolSklad.sumZkl; 
            end if; 
          end loop; 
 
          if(_mnozMjProCenaMj <> 0.0) then 
            _cenaMjPoriz = _sumZklProCenaMj / _mnozMjProCenaMj; 
          else 
            raise exception '_mnozMjProCenaMj je 0.0'; -- TODO vyresit co se bude dit 
          end if; 
          _cenaMj = _cenaMjPoriz + _polSklad.cenaMjNakl; 
          _sumZkl = _sumZkl + (_polSklad.mnozMj * _polSklad.cenaMjNakl); 
          --_sumZkl = _polSklad.mnozMj * _cenaMj; 
          -- ovsem pokud se jedna o prevodku musime zmenit cenu prijmky z prevodky 
          _updateCena = true; 
        end if; 
        _sumZklMen = 0.0; 
        select isdoklskladmenacizi(_rozsirenaPolozka.idDoklSkladPolozky, _polSklad.datVyst) into _isCiziMena; 
        if(_isCiziMena) then 
          _sumZklMen = _polSklad.mnozMj * _cenaMj; 
        end if; 
      else 
        _cenaMj = _skladKarta.prumCenaTuz; 
        _cenaMjPoriz = _skladKarta.prumCenaTuz; 
        if(_kurzDokladu is not null and _kurzDokladu <> 0.0) then 
          _cenaMj = _cenaMj / _kurzDokladu; 
        end if; 
        _sumZkl = _polSklad.mnozMj * _skladKarta.prumCenaTuz; 
        _sumZklMen = 0.0; 
        select isdoklskladmenacizi(_rozsirenaPolozka.idDoklSkladPolozky, _polSklad.datVyst) into _isCiziMena; 
        if(_isCiziMena) then 
          _sumZklMen = _polSklad.mnozMj * _cenaMj;  
        end if; 
      end if; 
 
      if (_updateCena) then 
        update dPolSklad set 
          cenaMjPoriz = _cenaMjPoriz, 
          cenaMj = _cenaMj, 
          cenaMjSkl = _cenaMjSkl, 
          sumZkl = _sumZkl, 
          sumZklMen = _sumZklMen 
          where idPolSklad = _rozsirenaPolozka.idPolSklad; 
      end if; 
 
      --execute sprepoctisumydokladu(_rozsirenaPolozka.idDoklSkladPolozky); 
 
      select sumZkl into _stavTuz from dpolsklad where idpolsklad = _rozsirenaPolozka.idPolSklad; 
    end if; 
  else 
    if(_polSklad.mnozMj = 0.0 and _rozsirenaPolozka.typpohybuk = 'typPohybu.prijem') then 
      _stavTuz = _polSklad.cenaMj * _kurzDokladu; 
    else 
      select sumZkl into _stavTuz from dpolsklad where idPolSklad = i_idrozsirenepolozky; 
    end if; 
  end if; 
 
  if(_rozsirenaPolozka.typpohybuk = 'typPohybu.vydej') then 
    _stavMj = _skladKarta.stavMj - (sign*_polSklad.mnozMj); 
    _stavTuz = _skladKarta.stavTuz - (sign*_stavTuz); 
  else 
    _stavMj = _skladKarta.stavMj + (sign*_polSklad.mnozMj); 
    _stavTuz = _skladKarta.stavTuz + (sign*_stavTuz); 
  end if; 
 
  if(_stavMj is null) then 
    raise exception 'stav mj je null a idpolsklad je % a _skladkarta je % a _polSklad je %',i_idrozsirenepolozky, _skladkarta, _polSklad; 
  end if; 
 
  _prumCenaTuz = _skladKarta.prumCenaTuz; 
  if(_stavMj <> 0.0) then 
    _prumCenaTuz = _stavTuz / _stavMj; 
  end if; 
  if(_prumCenaTuz is null) then 
    raise exception 'prumCenaTuz je null, _stavTuz je %, _stavMj je %', _stavTuz, _stavMj; 
  end if;  
  
  _datPosl = _skladKarta.datPosl; 
  if(_datPosl is null or _datPosl < _polSklad.datVyst) then 
    _datPosl = _polSklad.datVyst; 
  end if; 
 
  _poslCenaTuz = _skladKarta.poslCenaTuz; 
  _poslCenaMen = _skladKarta.poslCenaMen; 
  if(_rozsirenaPolozka.typpohybuk = 'typPohybu.prijem') then 
    _PoslCenaTuz := _polSklad.cenaMj * _KurzDokladu;    
    _PoslCenaMen := _polSklad.cenaMj;  
  end if; 
  
  _KurzSkladKarty := (select getKurzFromSklad(_skladKarta.idSkladu, _polSklad.datVyst)); 
  if (not _rozsirenaPolozka.storno) then 
    update skarty set  
      stavmj = _stavMj, 
      stavMjSPozadavky = _stavMj - _skladKarta.pozadavkyMj,
      stavTuz = _stavTuz, 
      stavMen = round(_stavTuz / _kurzSkladKarty, 2), 
      prumCenaTuz = _prumCenaTuz, 
      prumCenaMen = _prumCenaTuz / _kurzSkladKarty, 
      datPosl = _datPosl, 
      poslCenaTuz = _poslCenaTuz, 
      poslCenaMen = _poslCenaMen, 
      lastUpdate = LOCALTIMESTAMP 
      where idkarty = _rozsirenaPolozka.idSkladKarty; 
  end if; 
end; 
$BODY$
  LANGUAGE plpgsql VOLATILE
  COST 100;
ALTER FUNCTION skladprepoctirozsirenoupolozku(integer, integer) OWNER TO %firmaRole%;

-- Function: sprepoctiskladkartu(numeric, numeric, numeric, date, integer, integer, integer, integer)
-- DROP FUNCTION sprepoctiskladkartu(numeric, numeric, numeric, date, integer, integer, integer, integer);
CREATE OR REPLACE FUNCTION sprepoctiskladkartu(i_mnozmj numeric, i_cenamj numeric, i_sumzkl numeric, i_datvyst date, i_skladkartaid integer, i_doklskladid integer, i_skladid integer, i_nasobic integer)
  RETURNS void AS
$BODY$     
/* Prepocte stavy na skladove karte s id = iSkladKartaId */     
declare     
  /* Deklarace promennych */     
  _PomMnozMJ numeric(19,6);     
  _StavMJ numeric(19,6);     
  _StavTuz numeric(19,2);     
  _StavMen numeric(19,2);     
  _PrumCenaTuz numeric(19,6);     
  _PrumCenaMen numeric(19,6);     
  _PoslCenaTuz numeric(19,6);     
  _PoslCenaMen numeric(19,6);     
  _TypPohybuK varchar(50);     
  _DatPosl date;     
  _kurzSkladKarty numeric(19,6);    
  _kurzDokladu numeric(19,6);    
  loc_MnozMJ numeric(19,6);    
  _sklfifo boolean;     
  _pozadavkyMj numeric(19,6);    
    
  /* Deklarace kurzoru pro nacteni potrebnych dat ze skladove karty a dokladu */    
  declare ThisDoklSklad cursor for select TypPohybuK from dDoklSklad where idDoklSklad = i_DoklSkladId;    
  declare ThisSkladKarta cursor for select StavMJ,StavTuz,StavMen,PrumCenaTuz,PrumCenaMen,DatPosl,pozadavkyMj from sKarty where idKarty = i_SkladKartaId;      
begin    
  loc_MnozMJ:= i_MnozMJ;    
    
  /* Nacteni dat ze skladove karty */    
  open ThisSkladKarta;    
  fetch ThisSkladKarta into _StavMJ,    
    _StavTuz,_StavMen,_PrumCenaTuz,_PrumCenaMen,_DatPosl,_pozadavkyMj;    
  close ThisSkladKarta;    
      
  /* Zjisteni typPohybuK */    
  open ThisDoklSklad;    
  fetch ThisDoklSklad into _TypPohybuK;    
  close ThisDoklSklad;    
   
  /* zjisteni meny skladove karty */   
  _KurzSkladKarty := (select getKurzFromSklad(i_skladId, i_DatVyst));   
  _KurzDokladu := (select getKurzFromDoklad(i_doklSkladId, i_DatVyst));   
     
  /* pokud jde o vydej musim odecitat */    
  if _TypPohybuK = 'typPohybu.vydej' then    
    loc_MnozMJ:= 0-loc_MnozMJ;    
  end if;    
      
  /* Vypocet StavMJ */    
  _StavMJ:=_StavMJ+loc_MnozMJ*i_Nasobic;    
   
  /* Vypocet StavTuz */    
  if (loc_MnozMJ = 0 and _TypPohybuK = 'typPohybu.prijem')then     
    _PomMnozMJ:=1;    
    _StavTuz:=_StavTuz+(_PomMnozMJ*i_CenaMJ*i_Nasobic*_KurzDokladu); 
  else    
    _PomMnozMJ:=loc_MnozMJ; 
    if(_TypPohybuK = 'typPohybu.prijem') then 
       /* Pricitam sumZkl */ 
      _StavTuz:=_StavTuz+(i_sumZkl*i_Nasobic); 
    else 
      /* Odecitam sumZkl */ 
      _StavTuz:=_StavTuz-(i_sumZkl*i_Nasobic); 
    end if; 
  end if;    
   
  /* Vypocet StavMen */    
  _StavMen:=_StavMen+(_PomMnozMJ*i_CenaMJ*i_Nasobic*_KurzDokladu)/_KurzSkladKarty;    
   
  /* Vypocet prumernych cen ma byt proveden jen pri prijmu*/   
  if _TypPohybuK = 'typPohybu.prijem' then 
    /* Vypocet PrumCenaTuz */    
    if _StavMJ > 0 then    
      _PrumCenaTuz:=_StavTuz/_StavMJ;    
    end if;    
    /* Vypocet PrumCenaMen */    
    if _StavMJ > 0 then    
      _PrumCenaMen:=_StavMen/_StavMJ;    
    end if;    
  end if;    
    
  /* Prepocteni stavu skladu pro FIFO */ 
  -- drobna uprava prumerne ceny pokud by doslo k chybe vlivem zaokrouhleni a pro fifo nutnost   
  --_sklfifo := (select isfifosklad(i_DatVyst));   
  --if (_sklfifo) then   
    if(_StavMJ <> 0.0) then  
      _PrumCenaTuz := _StavTuz / _StavMJ;   
      _PrumCenaMen := _StavMen / _StavMJ;  
    end if;  
  --end if;   
    
  /* Nastaveni posledni ceny */    
  _PoslCenaTuz:= i_CenaMJ * _KurzDokladu;   
  _PoslCenaMen:= i_CenaMJ;    
   
  /* Nastaveni data posledniho pohybu na karte */    
  if _DatPosl is null then    
    _DatPosl:= i_DatVyst;    
  end if;    
  if _DatPosl < i_DatVyst then    
    _DatPosl:=i_DatVyst;    
  end if;    
   
  /* Update v databazi */ 
  if(_TypPohybuK = 'typPohybu.vydej') then 
    update sKarty set StavMJ = _StavMJ,     
      stavMjSPozadavky = _StavMJ - _pozadavkyMj,
      StavTuz = _StavTuz,     
      StavMen = _StavMen,     
      PrumCenaTuz = _PrumCenaTuz,     
      PrumCenaMen = _PrumCenaMen, 
      DatPosl = _DatPosl, 
      lastUpdate = LOCALTIMESTAMP 
      where     
      idKarty = i_SkladKartaId;     
  else 
    update sKarty set StavMJ = _StavMJ,     
      stavMjSPozadavky = _StavMJ - _pozadavkyMj,
      StavTuz = _StavTuz,     
      StavMen = _StavMen,     
      PrumCenaTuz = _PrumCenaTuz,     
      PrumCenaMen = _PrumCenaMen,     
      PoslCenaTuz = _PoslCenaTuz,     
      PoslCenaMen = _PoslcenaMen,     
      DatPosl = _DatPosl, 
      lastUpdate = LOCALTIMESTAMP 
      where     
      idKarty = i_SkladKartaId;     
  end if;   
end;    
$BODY$
  LANGUAGE plpgsql VOLATILE
  COST 100;
ALTER FUNCTION sprepoctiskladkartu(numeric, numeric, numeric, date, integer, integer, integer, integer) OWNER TO %firmaRole%;

-- Function: skladrealizujpozadavky()
-- DROP FUNCTION skladrealizujpozadavky();
CREATE OR REPLACE FUNCTION skladrealizujpozadavky()
  RETURNS void AS
$BODY$  
declare 
  _polSklad record; 
  _polSkladDoklSklad record; 
  _stavSkladu numeric(19,6); 
  _cenaMj numeric(19,6); 
  _pocetPozadavku integer; 
begin 
  for _polSklad in ( 
    select p.*, d.typPohybuK, d.stavSkladK from dpolsklad p join ddoklsklad d on p.idDoklSklad = d.idDoklSklad 
    where typPohybuK = 'typPohybu.vydej' and mnozMj = 0 and mnozMjPlan != 0 
    order by datSklad, idPolSklad) loop 
      --select dgetstavnasklade(_polSklad.idkarty, _polSklad.datSklad, /*_polSklad.idpolsklad*/ -1, _polSklad.sarze, _polSklad.expirace, _polSklad.typPohybuK) into _stavSkladu; 
      select stavMj, prumCenaTuz into _stavSkladu, _cenaMj from skarty where idkarty = _polSklad.idKarty; 
       
      if(_stavSkladu >= _polSklad.mnozMjPlan) then 
        update skarty set
          pozadavkyMj = pozadavkyMj - _polSklad.mnozMjPlan,
          stavMjSPozadavky = stavMjSPozadavky + _polSklad.mnozMjPlan
        where idkarty = _polSklad.idKarty;
        
        update dpolsklad set mnozMj = mnozMjPlan, cenaMj = _cenaMj, sumZkl = mnozMjPlan * _cenaMj, datSklad = LOCALTIMESTAMP where idpolsklad = _polSklad.idPolSklad; 
        perform sprepoctisumydokladu(_polSklad.iddoklsklad); 
         
        select count(*) into _pocetPozadavku from dpolSklad where idDoklSklad = _polSklad.idDoklSklad and mnozMj = 0.0 and mnozMjPlan != 0.0; 
        if(_pocetPozadavku = 0) then 
          if(compareVarchar(_polSklad.stavSkladK, 'stavSklad.vygenFaktNeuplny')) then 
            update ddoklsklad set stavSkladK = 'stavSklad.vygenFakt' where idDoklSklad = _polSklad.idDoklSklad; 
          end if; 
          if(compareVarchar(_polSklad.stavSkladK, 'stavSklad.vygenPoklNeuplny')) then 
            update ddoklsklad set stavSkladK = 'stavSklad.vygenPokl' where idDoklSklad = _polSklad.idDoklSklad; 
          end if; 
        end if; 
      end if; 
  end loop; 
end;     
$BODY$
  LANGUAGE plpgsql VOLATILE
  COST 100;
ALTER FUNCTION skladrealizujpozadavky() OWNER TO %firmaRole%;

CREATE OR REPLACE FUNCTION dabpolskladupdatedeletepozadavky()
  RETURNS trigger AS
$BODY$  
declare  
  _stavSkladu numeric(19,6); 
  _typPohybuK character varying; 
  _sklGenerovatPozadavky boolean; 
begin 
  if (tg_op = 'DELETE') then
    if(OLD.storno = false and OLD.mnozMj = 0.0 and OLD.mnozMjPlan <> 0.0) then
      -- pokud byla polozka smazana a nebyla stornovana a byla pozadavkem
      -- musime zaktualizovat mnozstvi pozadavku
      update skarty set
          pozadavkyMj = pozadavkyMj - OLD.mnozMjPlan,
          stavMjSPozadavky = stavMjSPozadavky + OLD.mnozMjPlan
        where idKarty = OLD.idKarty;
    end if;
    return OLD;
  else
    if(OLD.storno = false and NEW.storno = true and NEW.mnozMj = 0.0 and NEW.mnozMjPlan <> 0.0 and OLD.mnozMj = 0.0 and OLD.mnozMjPlan <> 0.0) then
      -- pokud byla polozka stornovana a byla pozadavkem
      -- musime zaktualizovat mnozstvi pozadavku
      update skarty set
          pozadavkyMj = pozadavkyMj - NEW.mnozMjPlan,
          stavMjSPozadavky = stavMjSPozadavky + NEW.mnozMjPlan
        where idKarty = NEW.idKarty;
    end if;
  end if;
  return NEW; 
end;  
$BODY$
  LANGUAGE plpgsql VOLATILE
  COST 100;
ALTER FUNCTION dabpolskladupdatedeletepozadavky() OWNER TO %firmaRole%;

CREATE TRIGGER dabpolskladupdatedeletepozadavky
  BEFORE UPDATE OR DELETE
  ON dpolsklad
  FOR EACH ROW
  EXECUTE PROCEDURE dabpolskladupdatedeletepozadavky();

-- zvyseni verze databaze
insert into dbversion(iddbversion, dbversion, needreimport, appversion, upgradetime)
  values (nextval('seq_dbversion'), 653, 't', %appVersion%, clock_timestamp());
