--------------------------------------------------------------------------------
-- Bug 30895 – [WinStrom] - Obecné vazby - umožnit vytvořit vazbu mezi libovolnými záznamy
--------------------------------------------------------------------------------

CREATE SEQUENCE seq_wvazbatyp
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;
ALTER TABLE public.seq_wvazbatyp OWNER TO %firmaRole%;

CREATE TABLE wvazbatyp (
  idvazbatyp integer DEFAULT nextval('seq_wvazbatyp'::regclass) NOT NULL,
  lastupdate timestamp without time zone NOT NULL DEFAULT '2008-01-01 00:00:00'::timestamp without time zone,
  kod character varying(20) NOT NULL,
  nazev character varying(255) NOT NULL,
  nazeva character varying(255),
  nazevb character varying(255),
  nazevc character varying(255),
  popis text,
  poznam text,
  show boolean NOT NULL DEFAULT true,
  platido integer NOT NULL DEFAULT 9999,
  platiod integer NOT NULL DEFAULT 0,
  automaticka boolean NOT NULL DEFAULT false,
  beankeysa text,
  beankeysb text,
  visiblea boolean NOT NULL DEFAULT true,
  visibleb boolean NOT NULL DEFAULT true,
  fknameb character varying(100),
  fkdbnameb character varying(100),
  tablenameb character varying(100),
  pkdbnameb character varying(100),
  modulb character varying(3),
  CONSTRAINT wvazbatyp_pkey PRIMARY KEY (idvazbatyp)
);
ALTER TABLE wvazbatyp OWNER TO %firmaRole%;

CREATE TABLE wkli_wvazbatyp (
  idvazbatyp integer NOT NULL,
  idvybklic integer NOT NULL,
  CONSTRAINT wkli_vazba_typ_pkey PRIMARY KEY (idvazbatyp, idvybklic),
  CONSTRAINT wkli_wvazbatyp_idvybklic FOREIGN KEY (idvybklic)
      REFERENCES wvybklice (idvybklice) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION,
  CONSTRAINT wkli_wvazbatyp_idvazbatyp FOREIGN KEY (idvazbatyp)
      REFERENCES wvazbatyp (idvazbatyp) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION
);
ALTER TABLE wkli_wvazbatyp OWNER TO %firmaRole%;

CREATE INDEX wkli_wvazbatyp_idvazbatyp_index ON wkli_wvazbatyp USING btree (idvazbatyp);
CREATE INDEX wkli_wvazbatyp_idvybklic_index ON wkli_wvazbatyp USING btree (idvybklic);


CREATE SEQUENCE seq_wvazba
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;
ALTER TABLE public.seq_wvazba OWNER TO %firmaRole%;

CREATE TABLE wvazba (
  idvazba integer DEFAULT nextval('seq_wvazba'::regclass) NOT NULL,
  lastupdate timestamp without time zone NOT NULL DEFAULT '2008-01-01 00:00:00'::timestamp without time zone,
  kod character varying(20) NOT NULL,
  nazev character varying(255) NOT NULL,
  nazeva character varying(255),
  nazevb character varying(255),
  nazevc character varying(255),
  popis text,
  poznam text,
  show boolean NOT NULL DEFAULT true,
  beankeya character varying(100),
  beankeyb character varying(100),
  idtaba integer DEFAULT 0,
  idtabb integer DEFAULT 0,
  visiblea boolean NOT NULL DEFAULT true,
  visibleb boolean NOT NULL DEFAULT true,
  tablenamea character varying(100),
  tablenameb character varying(100),
  pkdbnamea character varying(100),
  pkdbnameb character varying(100),
  idvazbatyp integer,
  CONSTRAINT wvazba_pkey PRIMARY KEY (idvazba),
  CONSTRAINT wvazba_idvazbatyp FOREIGN KEY (idvazbatyp)
      REFERENCES wvazbatyp (idvazbatyp) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION
);
ALTER TABLE wvazba OWNER TO %firmaRole%;

CREATE INDEX wVazba_IdVazbaTyp_index ON wVazba USING btree (IdVazbaTyp);

CREATE INDEX wvazba_beankeya_index ON wvazba USING btree (beankeya);
CREATE INDEX wvazba_beankeyb_index ON wvazba USING btree (beankeyb);
CREATE INDEX wvazba_idtaba_index ON wvazba USING btree (idtaba);
CREATE INDEX wvazba_idtabb_index ON wvazba USING btree (idtabb);


CREATE OR REPLACE FUNCTION existcolumn(_table varchar, _column varchar)
  RETURNS boolean AS $$
declare     
  _pocet integer;  
begin
  select count(*) into _pocet from pg_attribute as a where
    a.attname = _column and
    a.attrelid = (
        SELECT c.oid
        FROM pg_catalog.pg_class c
            LEFT JOIN pg_catalog.pg_namespace n ON n.oid = c.relnamespace
        WHERE c.relname ~ ('^('||lower(_table)||')$')
            AND pg_catalog.pg_table_is_visible(c.oid)
    );
  
  return (_pocet > 0);
end; 
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.existcolumn(varchar, varchar) OWNER TO %firmaRole%;

CREATE OR REPLACE FUNCTION wvazbadetail(_idvazba integer, _table varchar, _idcol varchar, _id integer)
  RETURNS varchar[] AS $$
declare
  dotaz varchar;
  zaznam record;
  _kod varchar;
  nazev varchar;
  modul varchar;
  datum date;
  castka numeric;
  
  dotazjoin varchar;
  polejoin varchar;
begin
  -- vraci kod; nazev; modul; castka; datum

  dotazjoin = '';
  polejoin = '';
  if lower(_table) in ('dpolfak', 'dpolint', 'dpolobch', 'dpolsklad') then
    dotazjoin = replace(' inner join ' || lower(_table) || ' as D on D.' || lower(_idcol) || ' = M.' || lower(_idcol), 'pol', 'dokl');
    polejoin = ', D.kod as koddoklad';
  end if;
  if lower(_table) = 'abanspoj' then
    dotazjoin = ' left outer join asmerkod as S on M.idsmerkod = S.idsmerkod';
    polejoin = ', S.kod as kodbanky';
  end if;
  if lower(_table) = 'anaklakt' then
    dotazjoin = ' left outer join atypnaklakt as T on M.idtypnaklakt = T.idtypnaklakt';
    polejoin = ', T.kod as typnakl';
  end if;
  if lower(_table) = 'pnepritomnost' then
    dotazjoin = ' left outer join posobahlav as O on M.idosoba = O.idosobahlav left outer join pcismzdsloz as S on M.idcismzdsloz = S.idcismzdsloz';
    polejoin = ', O.osbcis as osbcis, S.nazev as nazevSlozka';
  end if;
  if lower(_table) in ('ppracpomhlav', 'psmeny', 'psrazka') then
    dotazjoin = ' left outer join posobahlav as O on M.idosoba = O.idosobahlav';
    polejoin = ', O.osbcis as osbcis';
  end if;
  if lower(_table) = 'skarty' then
    dotazjoin = ' left outer join ccenik as C on M.idcenik = C.idcenik';
    polejoin = ', C.kod as kodcen';
  end if;
  dotaz = 'select M.*' || polejoin || ' from ' || _table || ' as M' || dotazjoin || ' where M.' || _idcol || ' = ' || _id;
  execute dotaz into zaznam;
  
  if zaznam is null then
    if _idvazba is not null then
      delete from wvazba where idvazba = _idvazba;
     end if;
    return array[null, null, null, null, null];
  end if;
  
  if lower(_table) in (
  'aadresar', 'apsc', 'askupfir', 'asmerkod', 'astaty', 'atypakt', 'atypnaklakt', 'ccenhlad', 'ccenik', 'cmerjed', 'cskupcen', 'cskupzboz', 'cstavcen', 
  'ctypatribut', 'ctypzbozi', 'dbsp', 'dformadopravy', 'dformauhrady', 'dformauhradycis', 'dkonsym', 'dpolsml', 'drady', 'dsmlouvy', 'dstavsml', 'dtypdokl', 
  'dtypsml', 'idodpodm', 'idruhdopr', 'iintrmerjed', 'ikrajurc', 'inomen', 'iobchtrans', 'izvlpoh', 'mleasing', 'mmajetek', 'msazby', 'mtypleas', 'mtypmaj', 
  'mumisteni', 'pcismzdsloz', 'pmzdcalc', 'pmzdcalchlav', 'pmzdcalchrom', 'pmzdcalcinit', 'pskuposob', 'pstalamzdsloz', 'ptyppracpom', 'sumisteni', 
  'ucinnost', 'uclendph', 'udphpren', 'umeny', 'usestavy', 'ustavzakazky', 'ustducty', 'ustrediska', 'utyporg', 'utypucop', 'utypzakazky', 'uucty', 
  'uvyhzakazky', 'uzakazky', 'wdotazy', 'wdotazyparam', 'wdotazyprop', 'wfiltry', 'wreport', 'wrole', 'wuzel', 'wvazba', 'wvazbatyp', 'wvybklice', 'pmzdsloz'
  ) then
    return array[zaznam.kod, zaznam.nazev, null, null, null];
  end if;
  
  if lower(_table) in ('ddoklfak', 'ddoklint', 'ddoklobch', 'ddoklsklad') then
    return array[zaznam.kod, zaznam.nazfirmy, zaznam.modul, cast(zaznam.sumcelkem as varchar), cast(zaznam.datvyst as varchar)];
  end if;
  
  if lower(_table) in ('dpolfak', 'dpolint', 'dpolobch', 'dpolsklad') then
    return array[zaznam.koddoklad, zaznam.nazev, zaznam.modul, cast(zaznam.sumzkl + zaznam.sumdph as varchar), cast(zaznam.datvyst as varchar)];
  end if;
  
  if lower(_table) = 'abanspoj' then
    if zaznam.kodbanky is not null then _kod = zaznam.kodbanky; else _kod = zaznam.bic; end if;
    if zaznam.buc is not null and zaznam.buc <> '' then nazev = zaznam.buc; else nazev = zaznam.iban; end if;
    return array[_kod, nazev, null, null, null];
  end if;
  
  if lower(_table) in ('akontakty', 'pdite', 'posobahlav', 'wuzivatel') then
    _kod = '';
    nazev = '';
    if zaznam.jmeno is not null then nazev = zaznam.jmeno; end if;
    if zaznam.prijmeni is not null then
      if length(nazev) > 0 then nazev = nazev || ' '; end if;
      nazev = nazev || zaznam.prijmeni; 
    end if;
    if lower(_table) = 'pdite' then
      datum = zaznam.datnaroz;
      if zaznam.rodcis is not null then _kod = zaznam.rodcis; end if;
    end if;
    if lower(_table) = 'posobahlav' then _kod = zaznam.osbcis; end if;
    if lower(_table) = 'wuzivatel' then _kod = zaznam.kod; end if;
    return array[_kod, nazev, null, null, cast(datum as varchar)];
  end if;
  
  if lower(_table) in ('amisturc', 'ckusovnik') then
    return array['', zaznam.nazev, null, null, null];
  end if;
  
  if lower(_table) = 'anaklakt' then
    return array[zaznam.typnakl, zaznam.poznam, null, null, null];
  end if;
  
  if lower(_table) = 'audalosti' then
    return array['', zaznam.predmet, null, null, null];
  end if;
  
  if lower(_table) = 'dpolprikazuhr' then
    return array['', zaznam.nazprijem, null, cast(zaznam.castka as varchar), null];
  end if;
  
  if lower(_table) = 'dprikazuhr' then
    return array['', zaznam.nazfirmy, null, cast(zaznam.celcastka as varchar), cast(zaznam.datvytvor as varchar)];
  end if;
  
  if lower(_table) = 'pnepritomnost' then
    return array[zaznam.osbcis, zaznam.nazevSlozka, null, null, cast(zaznam.platiod as varchar)];
  end if;
  if lower(_table) = 'ppracpomhlav' then
    return array[zaznam.osbcis, zaznam.kod, null, null, cast(zaznam.aktivniod as varchar)];
  end if;
  if lower(_table) = 'psmeny' then
    return array[zaznam.osbcis, null, null, null, '' || zaznam.rok || '-' || zaznam.mesic || '-01'];
  end if;
  if lower(_table) = 'psrazka' then
    return array[zaznam.osbcis, zaznam.kod, null, cast(zaznam.castka as varchar), cast(zaznam.srazkaod as varchar)];
  end if;
  
  if lower(_table) = 'skarty' then
    return array[zaznam.kodcen, zaznam.nazev, null, cast(zaznam.prumcenatuz as varchar), null];
  end if;
  
  if lower(_table) in ('wadrcert', 'wautcert', 'woscert') then
    return array[zaznam.certnazev, zaznam.uzivnazev, null, null, cast(zaznam.platiod as varchar)];
  end if;
  
  if lower(_table) = 'wpriloha' then
    return array['', zaznam.nazsoub, null, null, null];
  end if;
  
  if (existcolumn(_table, 'kod')) then
    _kod = zaznam.kod;
  else
    _kod = 'ID ' || _id;
  end if;
  
  if (existcolumn(_table, 'nazev')) then
    nazev = zaznam.nazev;
  else
    if (existcolumn(_table, 'nazfirmy')) then
      nazev = zaznam.nazfirmy;
    end if;
  end if;
  
  if (existcolumn(_table, 'modul')) then
    modul = zaznam.modul;
  end if;
  
  if (existcolumn(_table, 'sumcelkem')) then
    castka = zaznam.sumcelkem;
  end if;
  
  if (existcolumn(_table, 'datvyst')) then
    datum = zaznam.datvyst;
  end if;
  
  return array[_kod, nazev, modul, cast(castka as varchar), cast(datum as varchar)];
end; 
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.wvazbadetail(integer, varchar, varchar, integer) OWNER TO %firmaRole%;

create type vazbaautomat as (idvazbatyp integer, beanKey varchar, tableName varchar, pkName varchar, idRec integer);

CREATE OR REPLACE FUNCTION findvazbyautomat(_beankey varchar, _priId integer)
  RETURNS SETOF vazbaautomat AS $$
declare
  _typV record;
  _query varchar;
  _secId integer;
  _vazba vazbaautomat%rowtype;
begin
  for _typV in (select * from wvazbatyp where automaticka = true and beanKeysA = _beankey) loop
    _vazba.idvazbatyp = _typV.idvazbatyp;
    _vazba.beanKey = _typV.beanKeysB;
    _vazba.tableName = _typV.tableNameB;
    _vazba.pkName = _typV.pkDbNameB;

    _query = 'select ' || _typV.pkDbNameB || ' from ' || _typV.tableNameB || ' where ' || _typV.fkDbNameB || ' = ' || _priId;
    if _typV.modulB is not null then
      _query = _query || ' and modul = ''' || _typV.modulB || '''';
    end if;
    
    for _secId in execute _query loop
      _vazba.idRec = _secId;
      return next _vazba;
    end loop;
  end loop;
end;
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.findvazbyautomat(varchar, integer) OWNER TO %firmaRole%;

-- zvyseni verze databaze
insert into dbversion(iddbversion, dbversion, needreimport, appversion, upgradetime)
  values (nextval('seq_dbversion'), 662, 't', %appVersion%, clock_timestamp());
