--------------------------------------------------------------------------------
-- Bug 30669 – [WinStrom] - VF+PF - přeplatky (boži)
--------------------------------------------------------------------------------

ALTER TABLE nnastav
  ADD COLUMN idtdpreplphl integer,
  ADD COLUMN idtdpreplzav integer;

ALTER TABLE ONLY nnastav
    ADD CONSTRAINT nnastav_idtdpreplphl FOREIGN KEY (idtdpreplphl) REFERENCES dtypdokl(idtypdokl);
ALTER TABLE ONLY nnastav
    ADD CONSTRAINT nnastav_idtdpreplzav FOREIGN KEY (idtdpreplzav) REFERENCES dtypdokl(idtypdokl);

CREATE INDEX nnastav_idtdpreplphl_index ON nnastav USING btree (idtdpreplphl);
CREATE INDEX nnastav_idtdpreplzav_index ON nnastav USING btree (idtdpreplzav);


ALTER TABLE dtypdokl
  ADD COLUMN uhrauto boolean NOT NULL DEFAULT false;


ALTER TABLE ddoklfak
  ADD COLUMN juhsumpp numeric(15,2) DEFAULT 0.0 NOT NULL,
  ADD COLUMN juhsumppmen numeric(15,2) DEFAULT 0.0 NOT NULL,
  ADD COLUMN sumprepl numeric(15,2) DEFAULT 0.0 NOT NULL,
  ADD COLUMN sumpreplmen numeric(15,2) DEFAULT 0.0 NOT NULL;


--------------------------------------------------------------------------------


CREATE OR REPLACE FUNCTION dvazebtabinsdelpreplatek() RETURNS trigger
    AS $$
declare
  _doklFakId integer;
  _doklFakModul character varying;
  _sumPrepl numeric(15,2);
  _sumPreplMen numeric(15,2);
  _doklFak record;
  _domaciMena integer;
  _ciziMena boolean;
  ZAZNAM record;  
begin
  if (tg_op = 'DELETE') then  
    ZAZNAM := OLD;  
  else
    ZAZNAM := NEW;
  end if;

  if (tg_op = 'INSERT') then
    if (NEW.typVazbyK = 'typVazbyDokl.PP') then
      SELECT v.idtaba, v.modula INTO _doklFakId, _doklFakModul
      FROM dvazebtab AS v
      WHERE v.idtabb=ZAZNAM.idtabb AND v.modulb=ZAZNAM.modulb
        AND v.typvazbyk='typVazbyDokl.uhradaPP';

      --SELECT vazbaUhradaPP.idvazby, vazbaUhradaPP.typvazbyk, vazbaUhradaPP.castka as uhradapp,
      --  vazbaPP.idvazby, vazbaPP.typvazbyk, vazbaPP.castka as pp
      SELECT coalesce(sum(vazbaPP.castka), 0) INTO _sumPrepl
      FROM dvazebtab AS vazbaUhradaPP
      JOIN dvazebtab AS vazbaPP
        ON vazbaPP.idtabb=vazbaUhradaPP.idtabb AND vazbaPP.modulb=vazbaUhradaPP.modulb
          AND vazbaPP.typvazbyk='typVazbyDokl.PP' AND vazbaPP.storno=false
      WHERE vazbaUhradaPP.idtaba=_doklFakId AND vazbaUhradaPP.modula=_doklFakModul
        AND vazbaUhradaPP.typvazbyk='typVazbyDokl.uhradaPP' AND vazbaUhradaPP.storno=false;
    end if;
  else
    if (tg_op = 'DELETE' AND OLD.typVazbyK = 'typVazbyDokl.uhradaPP') then
      _doklFakId := OLD.idtaba;
      _doklFakModul := OLD.modula;

      --SELECT vazbaUhradaPP.idvazby, vazbaUhradaPP.typvazbyk, vazbaUhradaPP.castka as uhradapp,
      --  vazbaPP.idvazby, vazbaPP.typvazbyk, vazbaPP.castka as pp
      SELECT coalesce(sum(vazbaPP.castka), 0) INTO _sumPrepl
      FROM dvazebtab AS vazbaUhradaPP
      JOIN dvazebtab AS vazbaPP
        ON vazbaPP.idtabb=vazbaUhradaPP.idtabb AND vazbaPP.modulb=vazbaUhradaPP.modulb
          AND vazbaPP.typvazbyk='typVazbyDokl.PP' AND vazbaPP.storno=false
      WHERE vazbaUhradaPP.idtaba=_doklFakId AND vazbaUhradaPP.modula=_doklFakModul
        AND vazbaUhradaPP.typvazbyk='typVazbyDokl.uhradaPP' AND vazbaUhradaPP.storno=false
        AND vazbaUhradaPP.idvazby<>OLD.idvazby;
    end if;
  end if;

  SELECT * INTO _doklFak FROM ddoklfak WHERE iddoklfak=_doklFakId AND modul=_doklFakModul;

  SELECT idmeny INTO _domaciMena FROM nnastav 
  WHERE (platioddata is null or platioddata <= _doklFak.datVyst)
  ORDER BY coalesce(platioddata, '0001-01-01') DESC LIMIT 1;

  _ciziMena := (_doklFak.idMeny is not null and _domaciMena <> _doklFak.idMeny);
  if _ciziMena then
    _sumPreplMen := _sumPrepl;
    _sumPrepl := _sumPreplMen * _doklFak.kurz / _doklFak.kurzmnozstvi;
  else
    _sumPreplMen := 0;
  end if;

  UPDATE ddoklfak
  SET sumprepl=_sumPrepl, sumpreplmen=_sumPreplMen
  WHERE iddoklfak=_doklFakId AND modul=_doklFakModul;

  return NEW;
end;
$$
    LANGUAGE plpgsql;

ALTER FUNCTION public.dvazebtabinsdelpreplatek() OWNER TO %firmaRole%;


-- DROP TRIGGER dvazebtabinsdelpreplatek ON dvazebtab;
CREATE TRIGGER dvazebtabinsdelpreplatek
  AFTER INSERT OR DELETE
  ON dvazebtab
  FOR EACH ROW
  EXECUTE PROCEDURE dvazebtabinsdelpreplatek();


--------------------------------------------------------------------------------


CREATE OR REPLACE FUNCTION findvicevazeb(_inserttab character varying, _iddokl integer, _modul character varying, _uroven integer, _prechodvp boolean) RETURNS void
    AS $$ 
declare 
  tabulka varchar(20); 
  poleStav varchar(20); 
  dotaz varchar; 
  prechodPom varchar;   
begin 
  tabulka = getdoklnamepostfixbymodul(_modul); 
  prechodPom = ''; 
 
  if (_uroven = 0) then 
    if (tabulka = 'Fak') then 
      poleStav = 'stavUhrK'; 
    else 
      poleStav = 'stavUzivK';
      prechodPom = 'cast(null as numeric) as ';  
    end if; 
     
    dotaz =  
      'select 0 as idVazby, null as typVazbyK, false as storno, D.idDokl' || tabulka || ' as idDokl, D.modul, ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym,  
        D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.' || poleStav || ' as stavK,
        D.nazFirmy, D.mesto, ' || prechodPom || 'juhsum, ' || prechodPom || 'juhsummen
        from dDokl' || tabulka || ' as D where D.idDokl' || tabulka || ' = ' || _idDokl || ' and D.modul = ''' || _modul || ''''; 
  else 
 
  if (tabulka = 'Fak') then 
    dotaz =  
      'select V.idVazby, V.typVazbyK, V.storno, D.idDoklInt as idDokl, D.modul, ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym,  
        D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.stavUzivK as stavK,
        D.nazFirmy, D.mesto, cast(null as numeric) as juhsum, cast(null as numeric) as juhsummen
        from dVazebTab as V left outer join dDoklInt as D on V.idTabB=D.idDoklInt and V.modulB=D.modul  
        where V.idTabA=' || _idDokl || ' and V.modulA=''' || _modul || ''' and V.typVazbyK in (''typVazbyDokl.uhrada'', ''typVazbyDokl.uhradaKR'', 
            ''typVazbyDokl.uhradaZB'', ''typVazbyDokl.preceneni'', ''typVazbyDokl.vazbaZDD'', ''typVazbyDokl.uhradaZALOHY'', ''typVazbyDokl.zaucProdejky'')  
          and V.modulB in (''BAN'', ''POK'', ''INT'')  
      union 
      select V.idVazby, V.typVazbyK, V.storno, D.idDoklInt as idDokl, D.modul, ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym,  
        D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.stavUzivK as stavK,
        D.nazFirmy, D.mesto, cast(null as numeric) as juhsum, cast(null as numeric) as juhsummen
        from dVazebTab as V left outer join dDoklInt as D on V.idTabA=D.idDoklInt and V.modulA=D.modul  
        where V.idTabB=' || _idDokl || ' and V.modulB=''' || _modul || ''' and V.typVazbyK in (''typVazbyDokl.PP'')  
          and V.modulA in (''BAN'', ''POK'', ''INT'')  

      union 
      select V.idVazby, V.typVazbyK, V.storno, D.idDoklSklad as idDokl, D.modul, ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym,  
        D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.stavUzivK as stavK,
        D.nazFirmy, D.mesto, cast(null as numeric) as juhsum, cast(null as numeric) as juhsummen
        from dVazebTab as V left outer join dDoklSklad as D on V.idTabB=D.idDoklSklad and V.modulB=D.modul 
        where V.idTabA=' || _idDokl || ' and V.modulA=''' || _modul || ''' and V.typVazbyK = ''typVazbyDokl.hlavaSkl'' 
      union 
      select V.idVazby, V.typVazbyK, V.storno, D.idDoklFak as idDokl, D.modul, ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym,  
        D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.stavUzivK as stavK,
        D.nazFirmy, D.mesto, cast(null as numeric) as juhsum, cast(null as numeric) as juhsummen
        from dVazebTab as V left outer join dDoklFak as D on V.idTabB=D.idDoklFak and V.modulB=D.modul
        where V.idTabA=' || _idDokl || ' and V.modulA=''' || _modul || ''' and V.typVazbyK in (''typVazbyDokl.zaucProdejky'', ''typVazbyDokl.uhradaPP'')  
          and V.modulB in (''PHL'', ''ZAV'', ''FAP'', ''FAV'')
      union
      select V.idVazby, V.typVazbyK, V.storno, D.idDoklFak as idDokl, D.modul, ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym,  
        D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.stavUzivK as stavK,
        D.nazFirmy, D.mesto, cast(null as numeric) as juhsum, cast(null as numeric) as juhsummen
        from dVazebTab as V left outer join dDoklFak as D on V.idTabA=D.idDoklFak and V.modulA=D.modul
        where V.idTabB=' || _idDokl || ' and V.modulB=''' || _modul || ''' and V.typVazbyK in (''typVazbyDokl.zaucProdejky'', ''typVazbyDokl.uhradaPP'')  
          and V.modulA in (''PHL'', ''ZAV'', ''FAP'', ''FAV'')
      union  
      select min(V.idVazby) as idVazby, V.typVazbyK, V.storno, D.idDoklFak as idDokl, D.modul, ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym,  
        D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.stavUhrK as stavK,
        D.nazFirmy, D.mesto, D.juhsum, D.juhsummen
        from dVazebTab as V left outer join dDoklFak as D on V.idTabB=D.idDoklFak and V.modulB=D.modul 
        where V.idTabA in (select idPolFak from dPolFak where idDoklFak=' || _idDokl || ') and V.modulA=''' || _modul || ''' and 
          V.typVazbyK in (''typVazbyDokl.odpocetZALOHY'', ''typVazbyDokl.odpocetZDD'')  
        group by V.typVazbyK, V.storno, D.idDoklFak, D.modul, D.kod, D.varSym, D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.stavuhrK, D.nazFirmy, D.mesto, D.juhsum, D.juhsummen 
      union  
      select min(V.idVazby) as idVazby, V.typVazbyK, V.storno, D.idDoklFak as idDokl, D.modul, ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym,  
        D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.stavUhrK as stavK,
        D.nazFirmy, D.mesto, D.juhsum, D.juhsummen
        from dVazebTab as V left outer join dDoklFak as D on D.idDoklFak= 
                (select idDoklFak from dPolFak where idPolFak = V.idTabA) and V.modulA=D.modul 
        where V.idTabB=' || _idDokl || ' and V.modulB=''' || _modul || ''' and V.typVazbyK in (''typVazbyDokl.odpocetZALOHY'', ''typVazbyDokl.odpocetZDD'') 
        group by V.typVazbyK, V.storno, D.idDoklFak, D.modul, D.kod, D.varSym, D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.stavUhrK, D.nazFirmy, D.mesto, D.juhsum, D.juhsummen 
      union  
      select V.idVazby, V.typVazbyK, V.storno, D.idDoklObch as idDokl, D.modul, ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym,  
        D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.stavUzivK as stavK,
        D.nazFirmy, D.mesto, cast(null as numeric) as juhsum, cast(null as numeric) as juhsummen
        from dVazebTab as V left outer join dDoklObch as D on V.idTabA=D.idDoklObch and V.modulA=D.modul 
        where V.idTabB=' || _idDokl || ' and V.modulB=''' || _modul || ''' and V.typVazbyK in (''typVazbyDokl.obchod_faktura_hla'', ''typVazbyDokl.obchod_zaloha_hla'')'; 
  else if (tabulka = 'Int') then 
    dotaz =  
      'select V.idVazby, V.typVazbyK, V.storno, D.idDoklFak as idDokl, D.modul, ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym,  
        D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.stavUhrK as stavK,
        D.nazFirmy, D.mesto, D.juhsum, D.juhsummen
        from dVazebTab as V left outer join dDoklFak as D on V.idTabA=D.idDoklFak and V.modulA=D.modul 
        where V.idTabB=' || _idDokl || ' and V.modulB=''' || _modul || ''' and V.typVazbyK in (''typVazbyDokl.uhrada'', ''typVazbyDokl.uhradaKR'', 
            ''typVazbyDokl.uhradaZB'', ''typVazbyDokl.preceneni'', ''typVazbyDokl.vazbaZDD'', ''typVazbyDokl.uhradaZALOHY'', ''typVazbyDokl.zaucProdejky'')  
          and V.modulA in (''PHL'', ''ZAV'', ''FAP'', ''FAV'')
      union
      select V.idVazby, V.typVazbyK, V.storno, D.idDoklFak as idDokl, D.modul, ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym,  
        D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.stavUhrK as stavK,
        D.nazFirmy, D.mesto, D.juhsum, D.juhsummen
        from dVazebTab as V left outer join dDoklFak as D on V.idTabB=D.idDoklFak and V.modulB=D.modul 
        where V.idTabA=' || _idDokl || ' and V.modulA=''' || _modul || ''' and V.typVazbyK in (''typVazbyDokl.PP'')  
          and V.modulB in (''PHL'', ''ZAV'', ''FAP'', ''FAV'')
          
      union 
      select V.idVazby, V.typVazbyK, V.storno, D.idDoklSklad as idDokl, D.modul, ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym,  
        D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.stavUzivK as stavK,
        D.nazFirmy, D.mesto, cast(null as numeric) as juhsum, cast(null as numeric) as juhsummen
        from dVazebTab as V left outer join dDoklSklad as D on V.idTabB=D.idDoklSklad and V.modulB=D.modul 
        where V.idTabA=' || _idDokl || ' and V.modulA=''' || _modul || ''' and V.typVazbyK in (''typVazbyDokl.hlavaSkl'')  
      union 
      select V.idVazby, V.typVazbyK, V.storno, D.idDoklInt as idDokl, D.modul, ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym,  
        D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.stavUzivK as stavK,
        D.nazFirmy, D.mesto, cast(null as numeric) as juhsum, cast(null as numeric) as juhsummen
        from dVazebTab as V left outer join dDoklInt as D on V.idTabA=D.idDoklInt and V.modulA=D.modul 
        where V.idTabB=' || _idDokl || ' and V.modulB=''' || _modul || ''' and V.typVazbyK in (''typVazbyDokl.KR'', ''typVazbyDokl.ZB'')  
      union 
      select V.idVazby, V.typVazbyK, V.storno, D.idDoklInt as idDokl, D.modul, ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym,  
        D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.stavUzivK as stavK,
        D.nazFirmy, D.mesto, cast(null as numeric) as juhsum, cast(null as numeric) as juhsummen
        from dVazebTab as V left outer join dDoklInt as D on V.idTabB=D.idDoklInt and V.modulB=D.modul 
        where V.idTabA=' || _idDokl || ' and V.modulA=''' || _modul || ''' and V.typVazbyK in (''typVazbyDokl.KR'', ''typVazbyDokl.ZB'')'; 
  else if (tabulka = 'Sklad') then 
    dotaz =  
      'select V.idVazby, V.typVazbyK, V.storno, D.idDoklFak as idDokl, D.modul, ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym,  
        D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.stavUhrK as stavK,
        D.nazFirmy, D.mesto, D.juhsum, D.juhsummen
        from dVazebTab as V left outer join dDoklFak as D on V.idTabA=D.idDoklFak and V.modulA=D.modul 
        where V.idTabB=' || _idDokl || ' and V.modulB=''' || _modul || ''' and  
          V.modulA in (''FAV'',''FAP'',''PHL'',''ZAV'') and V.typVazbyK = ''typVazbyDokl.hlavaSkl''  
      union  
      select V.idVazby, V.typVazbyK, V.storno, D.idDoklInt as idDokl, D.modul, ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym,  
        D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.stavUzivK as stavK,
        D.nazFirmy, D.mesto, cast(null as numeric) as juhsum, cast(null as numeric) as juhsummen
        from dVazebTab as V left outer join dDoklInt as D on V.idTabA=D.idDoklInt and V.modulA=D.modul 
        where V.idTabB=' || _idDokl || ' and V.modulB=''' || _modul || ''' and  
          V.modulA in (''BAN'',''POK'',''INT'') and V.typVazbyK = ''typVazbyDokl.hlavaSkl'' 
      union  
      select V.idVazby, V.typVazbyK, V.storno, D.idDoklSklad as idDokl, D.modul, ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym,
        D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.stavUzivK as stavK,
        D.nazFirmy, D.mesto, cast(null as numeric) as juhsum, cast(null as numeric) as juhsummen
        from dVazebTab as V left outer join dDoklSklad as D on V.idTabA=D.idDoklSklad and V.modulA=D.modul
        where V.idTabB=' || _idDokl || ' and V.modulB=''' || _modul || ''' and
          V.modulA in (''SKL'') and V.typVazbyK = ''typVazbyDokl.hlavaSklKus''
      union
      select V.idVazby, V.typVazbyK, V.storno, D.idDoklSklad as idDokl, D.modul, ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym,
        D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.stavUzivK as stavK,
        D.nazFirmy, D.mesto, cast(null as numeric) as juhsum, cast(null as numeric) as juhsummen
        from dVazebTab as V left outer join dDoklSklad as D on V.idTabB=D.idDoklSklad and V.modulB=D.modul
        where V.idTabA=' || _idDokl || ' and V.modulA=''' || _modul || ''' and
          V.modulB in (''SKL'') and V.typVazbyK = ''typVazbyDokl.hlavaSklKus''
      union  
      select V.idVazby, V.typVazbyK, V.storno, D.idDoklSklad as idDokl, D.modul, ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym, 
        D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.stavUzivK as stavK,
        D.nazFirmy, D.mesto, cast(null as numeric) as juhsum, cast(null as numeric) as juhsummen
        from dVazebTab as V left outer join dDoklSklad as D on V.idTabA=D.idDoklSklad and V.modulA=D.modul 
        where V.idTabB=' || _idDokl || ' and V.modulB=''' || _modul || ''' and 
          V.modulA in (''SKL'') and V.typVazbyK = ''typVazbyDokl.hlavaSklPrevod'' 
      union 
      select V.idVazby, V.typVazbyK, V.storno, D.idDoklSklad as idDokl, D.modul, ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym, 
        D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.stavUzivK as stavK,
        D.nazFirmy, D.mesto, cast(null as numeric) as juhsum, cast(null as numeric) as juhsummen
        from dVazebTab as V left outer join dDoklSklad as D on V.idTabB=D.idDoklSklad and V.modulB=D.modul 
        where V.idTabA=' || _idDokl || ' and V.modulA=''' || _modul || ''' and 
          V.modulB in (''SKL'') and V.typVazbyK = ''typVazbyDokl.hlavaSklPrevod'' 
      union   
      select V.idVazby, V.typVazbyK, V.storno, D.idDoklObch as idDokl, D.modul, ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym,  
        D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.stavUzivK as stavK,
        D.nazFirmy, D.mesto, cast(null as numeric) as juhsum, cast(null as numeric) as juhsummen
        from dVazebTab as V left outer join dDoklObch as D on V.idTabA=D.idDoklObch and V.modulA=D.modul 
        where V.idTabB=' || _idDokl || ' and V.modulB=''' || _modul || ''' and V.typVazbyK = ''typVazbyDokl.obchod_sklad_hla'''; 
  else if (tabulka = 'Obch') then 
    if (_prechodVP = false and _modul in ('PPV', 'OBV')) then 
      prechodPom = ' and D.modul in (''PPV'', ''NAP'', ''OBV'')'; 
    end if; 
   
    dotaz =  
      'select V.idVazby, V.typVazbyK, V.storno, D.idDoklObch as idDokl, D.modul, ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym,  
        D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.stavUzivK as stavK,
        D.nazFirmy, D.mesto, cast(null as numeric) as juhsum, cast(null as numeric) as juhsummen
        from dVazebTab as V left outer join dDoklObch as D on V.idTabA=D.idDoklObch and V.modulA=D.modul  
        where V.idTabB=' || _idDokl || ' and V.modulB=''' || _modul || ''' and V.typVazbyK = ''typVazbyDokl.obchod_obchod_hla''' || prechodPom || ' 
      union  
      select V.idVazby, V.typVazbyK, V.storno, D.idDoklObch as idDokl, D.modul, ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym,  
        D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.stavUzivK as stavK,
        D.nazFirmy, D.mesto, cast(null as numeric) as juhsum, cast(null as numeric) as juhsummen
        from dVazebTab as V left outer join dDoklObch as D on V.idTabB=D.idDoklObch and V.modulB=D.modul  
        where V.idTabA=' || _idDokl || ' and V.modulA=''' || _modul || ''' and V.typVazbyK = ''typVazbyDokl.obchod_obchod_hla''' || prechodPom || ' 
      union 
      select V.idVazby, V.typVazbyK, V.storno, D.idDoklFak as idDokl, D.modul, ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym,  
        D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.stavUhrK as stavK,
        D.nazFirmy, D.mesto, D.juhsum, D.juhsummen
        from dVazebTab as V left outer join dDoklFak as D on V.idTabB=D.idDoklFak and V.modulB=D.modul  
        where V.idTabA=' || _idDokl || ' and V.modulA=''' || _modul || ''' and V.typVazbyK in (''typVazbyDokl.obchod_faktura_hla'', ''typVazbyDokl.obchod_zaloha_hla'') 
      union 
      select V.idVazby, V.typVazbyK, V.storno, D.idDoklSklad as idDokl, D.modul, ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym,  
        D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.stavUzivK as stavK,
        D.nazFirmy, D.mesto, cast(null as numeric) as juhsum, cast(null as numeric) as juhsummen
        from dVazebTab as V left outer join dDoklSklad as D on V.idTabB=D.idDoklSklad and V.modulB=D.modul  
        where V.idTabA=' || _idDokl || ' and V.modulA=''' || _modul || ''' and V.typVazbyK = ''typVazbyDokl.obchod_sklad_hla'''; 
  else 
    raise exception 'findVazbyDokladu(): Neznámá tabulka % pro modul %', tabulka, _modul;  
  end if; 
  end if; 
  end if; 
  end if; 
   
  end if; 
   
  execute 'insert into ' || _insertTab || ' (idvazebnidoklad, typVazbyK, storno, idDokl, modul, modulk, kod, varsym, datvyst, datucto, sumcelkem, 
  sumcelkemmen, popis, poznam, idtypdokl,idmeny, iduziv, stavk, nazFirmy, mesto, juhsum, juhsummen, uroven, novy) select *, ' || _uroven || ', true from ('  
     || dotaz || ') as UN where (select count(*) from ' || _insertTab || ' as TV where  
     (TV.idDokl = UN.idDokl and TV.modul = UN.modul) or UN.idVazby = TV.idvazebnidoklad) = 0'; 
end; 
$$
    LANGUAGE plpgsql;

ALTER FUNCTION public.findvicevazeb(_inserttab character varying, _iddokl integer, _modul character varying, _uroven integer, _prechodvp boolean) OWNER TO %firmaRole%;


-- zvyseni verze databaze
insert into dbversion(iddbversion, dbversion, needreimport, appversion, upgradetime)
  values (nextval('seq_dbversion'), 663, 't', %appVersion%, clock_timestamp());
