--------------------------------------------------------------------------------
-- Bug 31297 – [WinStrom] Odstranění nepoužívaných tabulek ve mzdách
--------------------------------------------------------------------------------

-- odstraneni generace odecitatelnych polozek
CREATE OR REPLACE FUNCTION pgeneratemzdslozforpracpomandmesic(_idpracpom integer, _rok integer, _mesic integer)
  RETURNS void AS $$
declare
  _datum date;
  _datumpo date;
  _idosoba integer;
  _change boolean;
begin
  _datum = date (_rok || '-' || _mesic || '-01');
  _datumpo = _datum + interval '1 month';
  
  select idosoba, pcantriggerchangeslozku(_rok, _mesic, idosoba) into _idosoba, _change from ppracpomhlav where idpracpomhlav = _idpracpom;

  -- vystupni slozky
  -- jiz se negeneruji, vytvari se pri vypoctu
  
  if (_change) then  
    -- stale slozky
    perform pupdatemzdslozfromstalasloz(pcreatemzdslozfromcis(idcismzdsloz, _idpracpom, _idosoba, _rok, _mesic), idstalamzdsloz)
      from pstalamzdsloz as S where idpracpom = _idpracpom
        and (datumod is null or (datumod < _datumpo)) and (datumdo is null or (datumdo >= _datum))
        and (select count(*) from pmzdsloz where rok = _rok and mesic = _mesic and idstalamzdsloz = S.idstalamzdsloz) = 0;
  
    -- srazky
    perform pupdatemzdslozfromsrazka(pcreatemzdslozfromcis(idcismzdsloz, null, _idosoba, _rok, _mesic), idsrazka)
      from psrazka as S where idosoba = _idosoba
        and (srazkaod is null or (srazkaod < _datumpo)) and (srazkado is null or (srazkado >= _datum))
        and (select count(*) from pmzdsloz where rok = _rok and mesic = _mesic and idsrazka = S.idsrazka) = 0;
  end if;
  
  -- smeny
  perform pgeneratesmenyforpracpom(_idpracpom, _rok, _mesic);
end; 
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.pgeneratemzdslozforpracpomandmesic(integer, integer, integer) OWNER TO %firmaRole%;


DROP TRIGGER podecitpolupdatemzdsloz ON podecitpol;
DROP TRIGGER podecitpoldeletemzdsloz ON podecitpol;

DROP FUNCTION podecitpolupdatemzdsloz();
DROP FUNCTION podecitpoldeletemzdsloz();

DROP FUNCTION pupdatemzdslozfromodecitpol(integer, integer);

ALTER TABLE pmzdsloz DROP COLUMN idodecitpol;

DROP TABLE pduchod;
DROP TABLE podecitpol;

DROP SEQUENCE seq_pduchod;
DROP SEQUENCE seq_podecitpol;


--------------------------------------------------------------------------------
-- Bug 31200 – winstrom mzdy - oznámení o nástupu - skončení zaměstnání
--------------------------------------------------------------------------------
ALTER TABLE posoba ADD COLUMN zahranicniduch boolean DEFAULT false;
ALTER TABLE posoba ADD COLUMN duchodod date;

-- zvyseni verze databaze
insert into dbversion(iddbversion, dbversion, needreimport, appversion, upgradetime)
  values (nextval('seq_dbversion'), 664, 't', %appVersion%, clock_timestamp());
