--------------------------------------------------------------------------------
-- Bug 34166 – [FlexiBee] Rozšířit datový model kvůli workflow
--------------------------------------------------------------------------------

create table ACT_GE_PROPERTY (
    NAME_ varchar(64),
    VALUE_ varchar(300),
    REV_ integer,
    primary key (NAME_)
);

insert into ACT_GE_PROPERTY
values ('schema.version', '5.12', 1);

insert into ACT_GE_PROPERTY
values ('schema.history', 'create(5.12)', 1);

insert into ACT_GE_PROPERTY
values ('next.dbid', '1', 1);

create table ACT_GE_BYTEARRAY (
    ID_ varchar(64),
    REV_ integer,
    NAME_ varchar(255),
    DEPLOYMENT_ID_ varchar(64),
    BYTES_ bytea,
    GENERATED_ boolean,
    primary key (ID_)
);

create table ACT_RE_DEPLOYMENT (
    ID_ varchar(64),
    NAME_ varchar(255),
    CATEGORY_ varchar(255),
    DEPLOY_TIME_ timestamp,
    primary key (ID_)
);

create table ACT_RE_MODEL (
    ID_ varchar(64) not null,
    REV_ integer,
    NAME_ varchar(255),
    KEY_ varchar(255),
    CATEGORY_ varchar(255),
    CREATE_TIME_ timestamp,
    LAST_UPDATE_TIME_ timestamp,
    VERSION_ integer,
    META_INFO_ varchar(4000),
    DEPLOYMENT_ID_ varchar(64),
    EDITOR_SOURCE_VALUE_ID_ varchar(64),
    EDITOR_SOURCE_EXTRA_VALUE_ID_ varchar(64),
    primary key (ID_)
);

create table ACT_RU_EXECUTION (
    ID_ varchar(64),
    REV_ integer,
    PROC_INST_ID_ varchar(64),
    BUSINESS_KEY_ varchar(255),
    PARENT_ID_ varchar(64),
    PROC_DEF_ID_ varchar(64),
    SUPER_EXEC_ varchar(64),
    ACT_ID_ varchar(255),
    IS_ACTIVE_ boolean,
    IS_CONCURRENT_ boolean,
    IS_SCOPE_ boolean,
  IS_EVENT_SCOPE_ boolean,
  SUSPENSION_STATE_ integer,
  CACHED_ENT_STATE_ integer,
    primary key (ID_),
    unique (PROC_DEF_ID_, BUSINESS_KEY_)
);

create table ACT_RU_JOB (
    ID_ varchar(64) NOT NULL,
    REV_ integer,
    TYPE_ varchar(255) NOT NULL,
    LOCK_EXP_TIME_ timestamp,
    LOCK_OWNER_ varchar(255),
    EXCLUSIVE_ boolean,
    EXECUTION_ID_ varchar(64),
    PROCESS_INSTANCE_ID_ varchar(64),
    PROC_DEF_ID_ varchar(64),
    RETRIES_ integer,
    EXCEPTION_STACK_ID_ varchar(64),
    EXCEPTION_MSG_ varchar(4000),
    DUEDATE_ timestamp,
    REPEAT_ varchar(255),
    HANDLER_TYPE_ varchar(255),
    HANDLER_CFG_ varchar(4000),
    primary key (ID_)
);

create table ACT_RE_PROCDEF (
    ID_ varchar(64) NOT NULL,
    REV_ integer,
    CATEGORY_ varchar(255),
    NAME_ varchar(255),
    KEY_ varchar(255) NOT NULL,
    VERSION_ integer NOT NULL,
    DEPLOYMENT_ID_ varchar(64),
    RESOURCE_NAME_ varchar(4000),
    DGRM_RESOURCE_NAME_ varchar(4000),
    DESCRIPTION_ varchar(4000),
    HAS_START_FORM_KEY_ boolean,
    SUSPENSION_STATE_ integer,
    primary key (ID_)
);

create table ACT_RU_TASK (
    ID_ varchar(64),
    REV_ integer,
    EXECUTION_ID_ varchar(64),
    PROC_INST_ID_ varchar(64),
    PROC_DEF_ID_ varchar(64),
    NAME_ varchar(255),
    PARENT_TASK_ID_ varchar(64),
    DESCRIPTION_ varchar(4000),
    TASK_DEF_KEY_ varchar(255),
    OWNER_ varchar(255),
    ASSIGNEE_ varchar(255),
    DELEGATION_ varchar(64),
    PRIORITY_ integer,
    CREATE_TIME_ timestamp,
    DUE_DATE_ timestamp,
    SUSPENSION_STATE_ integer,
    primary key (ID_)
);

create table ACT_RU_IDENTITYLINK (
    ID_ varchar(64),
    REV_ integer,
    GROUP_ID_ varchar(255),
    TYPE_ varchar(255),
    USER_ID_ varchar(255),
    TASK_ID_ varchar(64),
    PROC_INST_ID_ varchar(64),
    PROC_DEF_ID_ varchar (64),
    primary key (ID_)
);

create table ACT_RU_VARIABLE (
    ID_ varchar(64) not null,
    REV_ integer,
    TYPE_ varchar(255) not null,
    NAME_ varchar(255) not null,
    EXECUTION_ID_ varchar(64),
    PROC_INST_ID_ varchar(64),
    TASK_ID_ varchar(64),
    BYTEARRAY_ID_ varchar(64),
    DOUBLE_ double precision,
    LONG_ bigint,
    TEXT_ varchar(4000),
    TEXT2_ varchar(4000),
    primary key (ID_)
);

create table ACT_RU_EVENT_SUBSCR (
    ID_ varchar(64) not null,
    REV_ integer,
    EVENT_TYPE_ varchar(255) not null,
    EVENT_NAME_ varchar(255),
    EXECUTION_ID_ varchar(64),
    PROC_INST_ID_ varchar(64),
    ACTIVITY_ID_ varchar(64),
    CONFIGURATION_ varchar(255),
    CREATED_ timestamp not null,
    primary key (ID_)
);

create index ACT_IDX_EXEC_BUSKEY on ACT_RU_EXECUTION(BUSINESS_KEY_);
create index ACT_IDX_TASK_CREATE on ACT_RU_TASK(CREATE_TIME_);
create index ACT_IDX_IDENT_LNK_USER on ACT_RU_IDENTITYLINK(USER_ID_);
create index ACT_IDX_IDENT_LNK_GROUP on ACT_RU_IDENTITYLINK(GROUP_ID_);
create index ACT_IDX_EVENT_SUBSCR_CONFIG_ on ACT_RU_EVENT_SUBSCR(CONFIGURATION_);
create index ACT_IDX_VARIABLE_TASK_ID on ACT_RU_VARIABLE(TASK_ID_);

create index ACT_IDX_BYTEAR_DEPL on ACT_GE_BYTEARRAY(DEPLOYMENT_ID_);
alter table ACT_GE_BYTEARRAY
    add constraint ACT_FK_BYTEARR_DEPL
    foreign key (DEPLOYMENT_ID_) 
    references ACT_RE_DEPLOYMENT (ID_);

alter table ACT_RE_PROCDEF
    add constraint ACT_UNIQ_PROCDEF
    unique (KEY_,VERSION_);
    
create index ACT_IDX_EXE_PROCINST on ACT_RU_EXECUTION(PROC_INST_ID_);
alter table ACT_RU_EXECUTION
    add constraint ACT_FK_EXE_PROCINST 
    foreign key (PROC_INST_ID_) 
    references ACT_RU_EXECUTION (ID_);

create index ACT_IDX_EXE_PARENT on ACT_RU_EXECUTION(PARENT_ID_);
alter table ACT_RU_EXECUTION
    add constraint ACT_FK_EXE_PARENT
    foreign key (PARENT_ID_) 
    references ACT_RU_EXECUTION (ID_);
    
create index ACT_IDX_EXE_SUPER on ACT_RU_EXECUTION(SUPER_EXEC_);
alter table ACT_RU_EXECUTION
    add constraint ACT_FK_EXE_SUPER
    foreign key (SUPER_EXEC_) 
    references ACT_RU_EXECUTION (ID_);

create index ACT_IDX_EXE_PROCDEF on ACT_RU_EXECUTION(PROC_DEF_ID_); 
alter table ACT_RU_EXECUTION
    add constraint ACT_FK_EXE_PROCDEF 
    foreign key (PROC_DEF_ID_) 
    references ACT_RE_PROCDEF (ID_);    
    

create index ACT_IDX_TSKASS_TASK on ACT_RU_IDENTITYLINK(TASK_ID_);
alter table ACT_RU_IDENTITYLINK
    add constraint ACT_FK_TSKASS_TASK
    foreign key (TASK_ID_) 
    references ACT_RU_TASK (ID_);
    
create index ACT_IDX_ATHRZ_PROCEDEF on ACT_RU_IDENTITYLINK(PROC_DEF_ID_);
alter table ACT_RU_IDENTITYLINK
    add constraint ACT_FK_ATHRZ_PROCEDEF
    foreign key (PROC_DEF_ID_) 
    references ACT_RE_PROCDEF (ID_);
    
create index ACT_IDX_IDL_PROCINST on ACT_RU_IDENTITYLINK(PROC_INST_ID_);
alter table ACT_RU_IDENTITYLINK
    add constraint ACT_FK_IDL_PROCINST
    foreign key (PROC_INST_ID_) 
    references ACT_RU_EXECUTION (ID_);    
    
create index ACT_IDX_TASK_EXEC on ACT_RU_TASK(EXECUTION_ID_);
alter table ACT_RU_TASK
    add constraint ACT_FK_TASK_EXE
    foreign key (EXECUTION_ID_)
    references ACT_RU_EXECUTION (ID_);
    
create index ACT_IDX_TASK_PROCINST on ACT_RU_TASK(PROC_INST_ID_);
alter table ACT_RU_TASK
    add constraint ACT_FK_TASK_PROCINST
    foreign key (PROC_INST_ID_)
    references ACT_RU_EXECUTION (ID_);
    
create index ACT_IDX_TASK_PROCDEF on ACT_RU_TASK(PROC_DEF_ID_);
alter table ACT_RU_TASK
  add constraint ACT_FK_TASK_PROCDEF
  foreign key (PROC_DEF_ID_)
  references ACT_RE_PROCDEF (ID_);
  
create index ACT_IDX_VAR_EXE on ACT_RU_VARIABLE(EXECUTION_ID_);
alter table ACT_RU_VARIABLE 
    add constraint ACT_FK_VAR_EXE
    foreign key (EXECUTION_ID_) 
    references ACT_RU_EXECUTION (ID_);

create index ACT_IDX_VAR_PROCINST on ACT_RU_VARIABLE(PROC_INST_ID_);
alter table ACT_RU_VARIABLE
    add constraint ACT_FK_VAR_PROCINST
    foreign key (PROC_INST_ID_)
    references ACT_RU_EXECUTION(ID_);

create index ACT_IDX_VAR_BYTEARRAY on ACT_RU_VARIABLE(BYTEARRAY_ID_);
alter table ACT_RU_VARIABLE 
    add constraint ACT_FK_VAR_BYTEARRAY 
    foreign key (BYTEARRAY_ID_) 
    references ACT_GE_BYTEARRAY (ID_);

create index ACT_IDX_JOB_EXCEPTION on ACT_RU_JOB(EXCEPTION_STACK_ID_);
alter table ACT_RU_JOB 
    add constraint ACT_FK_JOB_EXCEPTION
    foreign key (EXCEPTION_STACK_ID_) 
    references ACT_GE_BYTEARRAY (ID_);

create index ACT_IDX_EVENT_SUBSCR on ACT_RU_EVENT_SUBSCR(EXECUTION_ID_);
alter table ACT_RU_EVENT_SUBSCR
    add constraint ACT_FK_EVENT_EXEC
    foreign key (EXECUTION_ID_)
    references ACT_RU_EXECUTION(ID_);

create index ACT_IDX_MODEL_SOURCE on ACT_RE_MODEL(EDITOR_SOURCE_VALUE_ID_);
alter table ACT_RE_MODEL 
    add constraint ACT_FK_MODEL_SOURCE 
    foreign key (EDITOR_SOURCE_VALUE_ID_) 
    references ACT_GE_BYTEARRAY (ID_);

create index ACT_IDX_MODEL_SOURCE_EXTRA on ACT_RE_MODEL(EDITOR_SOURCE_EXTRA_VALUE_ID_);
alter table ACT_RE_MODEL 
    add constraint ACT_FK_MODEL_SOURCE_EXTRA 
    foreign key (EDITOR_SOURCE_EXTRA_VALUE_ID_) 
    references ACT_GE_BYTEARRAY (ID_);
    
create index ACT_IDX_MODEL_DEPLOYMENT on ACT_RE_MODEL(DEPLOYMENT_ID_);    
alter table ACT_RE_MODEL 
    add constraint ACT_FK_MODEL_DEPLOYMENT 
    foreign key (DEPLOYMENT_ID_) 
    references ACT_RE_DEPLOYMENT (ID_);        

create table ACT_HI_PROCINST (
    ID_ varchar(64) not null,
    PROC_INST_ID_ varchar(64) not null,
    BUSINESS_KEY_ varchar(255),
    PROC_DEF_ID_ varchar(64) not null,
    START_TIME_ timestamp not null,
    END_TIME_ timestamp,
    DURATION_ bigint,
    START_USER_ID_ varchar(255),
    START_ACT_ID_ varchar(255),
    END_ACT_ID_ varchar(255),
    SUPER_PROCESS_INSTANCE_ID_ varchar(64),
    DELETE_REASON_ varchar(4000),
    primary key (ID_),
    unique (PROC_INST_ID_),
    unique (PROC_DEF_ID_, BUSINESS_KEY_)
);

create table ACT_HI_ACTINST (
    ID_ varchar(64) not null,
    PROC_DEF_ID_ varchar(64) not null,
    PROC_INST_ID_ varchar(64) not null,
    EXECUTION_ID_ varchar(64) not null,
    ACT_ID_ varchar(255) not null,
    TASK_ID_ varchar(64),
    CALL_PROC_INST_ID_ varchar(64),
    ACT_NAME_ varchar(255),
    ACT_TYPE_ varchar(255) not null,
    ASSIGNEE_ varchar(64),
    START_TIME_ timestamp not null,
    END_TIME_ timestamp,
    DURATION_ bigint,
    primary key (ID_)
);

create table ACT_HI_TASKINST (
    ID_ varchar(64) not null,
    PROC_DEF_ID_ varchar(64),
    TASK_DEF_KEY_ varchar(255),
    PROC_INST_ID_ varchar(64),
    EXECUTION_ID_ varchar(64),
    NAME_ varchar(255),
    PARENT_TASK_ID_ varchar(64),
    DESCRIPTION_ varchar(4000),
    OWNER_ varchar(255),
    ASSIGNEE_ varchar(255),
    START_TIME_ timestamp not null,
    CLAIM_TIME_ timestamp,
    END_TIME_ timestamp,
    DURATION_ bigint,
    DELETE_REASON_ varchar(4000),
    PRIORITY_ integer,
    DUE_DATE_ timestamp,
    FORM_KEY_ varchar(255),
    primary key (ID_)
);

create table ACT_HI_VARINST (
    ID_ varchar(64) not null,
    PROC_INST_ID_ varchar(64),
    EXECUTION_ID_ varchar(64),
    TASK_ID_ varchar(64),
    NAME_ varchar(255) not null,
    VAR_TYPE_ varchar(100),
    REV_ integer,
    BYTEARRAY_ID_ varchar(64),
    DOUBLE_ double precision,
    LONG_ bigint,
    TEXT_ varchar(4000),
    TEXT2_ varchar(4000),
    primary key (ID_)
);

create table ACT_HI_DETAIL (
    ID_ varchar(64) not null,
    TYPE_ varchar(255) not null,
    PROC_INST_ID_ varchar(64),
    EXECUTION_ID_ varchar(64),
    TASK_ID_ varchar(64),
    ACT_INST_ID_ varchar(64),
    NAME_ varchar(255) not null,
    VAR_TYPE_ varchar(64),
    REV_ integer,
    TIME_ timestamp not null,
    BYTEARRAY_ID_ varchar(64),
    DOUBLE_ double precision,
    LONG_ bigint,
    TEXT_ varchar(4000),
    TEXT2_ varchar(4000),
    primary key (ID_)
);

create table ACT_HI_COMMENT (
    ID_ varchar(64) not null,
    TYPE_ varchar(255),
    TIME_ timestamp not null,
    USER_ID_ varchar(255),
    TASK_ID_ varchar(64),
    PROC_INST_ID_ varchar(64),
    ACTION_ varchar(255),
    MESSAGE_ varchar(4000),
    FULL_MSG_ bytea,
    primary key (ID_)
);

create table ACT_HI_ATTACHMENT (
    ID_ varchar(64) not null,
    REV_ integer,
    USER_ID_ varchar(255),
    NAME_ varchar(255),
    DESCRIPTION_ varchar(4000),
    TYPE_ varchar(255),
    TASK_ID_ varchar(64),
    PROC_INST_ID_ varchar(64),
    URL_ varchar(4000),
    CONTENT_ID_ varchar(64),
    primary key (ID_)
);


create index ACT_IDX_HI_PRO_INST_END on ACT_HI_PROCINST(END_TIME_);
create index ACT_IDX_HI_PRO_I_BUSKEY on ACT_HI_PROCINST(BUSINESS_KEY_);
create index ACT_IDX_HI_ACT_INST_START on ACT_HI_ACTINST(START_TIME_);
create index ACT_IDX_HI_ACT_INST_END on ACT_HI_ACTINST(END_TIME_);
create index ACT_IDX_HI_DETAIL_PROC_INST on ACT_HI_DETAIL(PROC_INST_ID_);
create index ACT_IDX_HI_DETAIL_ACT_INST on ACT_HI_DETAIL(ACT_INST_ID_);
create index ACT_IDX_HI_DETAIL_TIME on ACT_HI_DETAIL(TIME_);
create index ACT_IDX_HI_DETAIL_NAME on ACT_HI_DETAIL(NAME_);
create index ACT_IDX_HI_DETAIL_TASK_ID on ACT_HI_DETAIL(TASK_ID_);
create index ACT_IDX_HI_PROCVAR_PROC_INST on ACT_HI_VARINST(PROC_INST_ID_);
create index ACT_IDX_HI_PROCVAR_NAME_TYPE on ACT_HI_VARINST(NAME_, VAR_TYPE_);
create index ACT_IDX_HI_ACT_INST_PROCINST on ACT_HI_ACTINST(PROC_INST_ID_, ACT_ID_);
create index ACT_IDX_HI_ACT_INST_EXEC on ACT_HI_ACTINST(EXECUTION_ID_, ACT_ID_);



-- identity
CREATE OR REPLACE view act_id_group AS
select kod as id_, 0 as rev_, nazev as name_, text 'assignment' as type_ from wactskupina where typrolek = 'typRole.glo';

CREATE OR REPLACE view act_id_membership AS
select wactskupina.kod as group_id_, wuzivatel.kod as user_id_ from wactclenstvi inner join wuzivatel on wuzivatel.iduzivatel = wactclenstvi.iduzivatel inner join wactskupina on wactskupina.idactskupina = wactclenstvi.idactskupina where wactskupina.typrolek = 'typRole.glo';

CREATE OR REPLACE view act_id_user AS
select kod as id_, 0 as rev_, jmeno as first_, prijmeni as last_, email as email_, text '' as pwd_, text '' as picture_id_ from wuzivatel;

create table ACT_ID_INFO (
    ID_ varchar(64),
    REV_ integer,
    USER_ID_ varchar(64),
    TYPE_ varchar(64),
    KEY_ varchar(255),
    VALUE_ varchar(255),
    PASSWORD_ bytea,
    PARENT_ID_ varchar(255),
    primary key (ID_)
);


CREATE OR REPLACE FUNCTION dactiviticreatetask()
  RETURNS trigger AS
$BODY$  
declare 
    _idfirmy integer;
    _idzakazky integer;
    _iddoklfak integer; 
    _iddoklint integer;
    _iddoklobch integer;
    _iddoklsklad integer;
    _idcenik integer;
    _idkontakt integer;
    _idkonektor integer; 
    _idreport integer;
    _idsmlouvy integer; 
    _idpolsml integer;

begin
    SELECT long_ INTO _idfirmy FROM act_ru_variable where execution_id_ = NEW.execution_id_ and proc_inst_id_ = NEW.proc_inst_id_ and name_ = 'Adresar' limit 1;
    SELECT long_ INTO _idzakazky FROM act_ru_variable where execution_id_ = NEW.execution_id_ and proc_inst_id_ = NEW.proc_inst_id_ and name_ = 'Zakazka' limit 1;
    SELECT long_ INTO _iddoklfak FROM act_ru_variable where execution_id_ = NEW.execution_id_ and proc_inst_id_ = NEW.proc_inst_id_ and name_ = 'DoklFak' limit 1;
    SELECT long_ INTO _iddoklint FROM act_ru_variable where execution_id_ = NEW.execution_id_ and proc_inst_id_ = NEW.proc_inst_id_ and name_ = 'DoklInt' limit 1;
    SELECT long_ INTO _iddoklobch FROM act_ru_variable where execution_id_ = NEW.execution_id_ and proc_inst_id_ = NEW.proc_inst_id_ and name_ = 'DoklObch' limit 1;
    SELECT long_ INTO _iddoklsklad FROM act_ru_variable where execution_id_ = NEW.execution_id_ and proc_inst_id_ = NEW.proc_inst_id_ and name_ = 'PolSklad' limit 1;
    SELECT long_ INTO _idkontakt FROM act_ru_variable where execution_id_ = NEW.execution_id_ and proc_inst_id_ = NEW.proc_inst_id_ and name_ = 'Kontakt' limit 1;
    SELECT long_ INTO _idkonektor FROM act_ru_variable where execution_id_ = NEW.execution_id_ and proc_inst_id_ = NEW.proc_inst_id_ and name_ = 'Konektor' limit 1;
    SELECT long_ INTO _idreport FROM act_ru_variable where execution_id_ = NEW.execution_id_ and proc_inst_id_ = NEW.proc_inst_id_ and name_ = 'Report' limit 1;
    SELECT long_ INTO _idsmlouvy FROM act_ru_variable where execution_id_ = NEW.execution_id_ and proc_inst_id_ = NEW.proc_inst_id_ and name_ = 'Smlouva' limit 1;
    SELECT long_ INTO _idpolsml FROM act_ru_variable where execution_id_ = NEW.execution_id_ and proc_inst_id_ = NEW.proc_inst_id_ and name_ = 'PolSmlouvy' limit 1;

    INSERT INTO audalosti (druhudalk, celodenni, termin, predmet, prioritak, stavudalk, zahajeni, popis, idtypakt, iduzivatel, taskDefinitionKey, processDefinitionId, actRuTask, idfirmy, idzakazky, iddoklfak, iddoklint, iddoklobch, iddoklsklad, idcenik, idkontakt, idkonektor, idreport, idsmlouvy, idpolsml
    ) 
    VALUES (
        'druhUdal.workflow', 
        false, 
        date_trunc('minute', NEW.due_date_), 
        NEW.name_, 
        CASE 
            WHEN NEW.priority_ >= 80 THEN 'priorita.kriticka'
            WHEN NEW.priority_ >= 60 THEN 'priorita.vysoka'
            WHEN NEW.priority_ >= 40 THEN 'priorita.stredni'
            WHEN NEW.priority_ >= 20 THEN 'priorita.nizka'
            ELSE 'priorita.velmiNizka' 
        END,
        'stavUdal.nove',
        date_trunc('minute', NEW.create_time_),
        NEW.description_,
        (select idtypakt from atypakt where druhudalk = 'druhUdal.workflow' limit 1),
        (select iduzivatel from wuzivatel where kod = NEW.assignee_ limit 1),
        NEW.task_def_key_,
        NEW.proc_def_id_,
        NEW.id_,
        _idfirmy, 
        _idzakazky, 
        _iddoklfak, 
        _iddoklint, 
        _iddoklobch, 
        _iddoklsklad, 
        _idcenik, 
        _idkontakt, 
        _idkonektor, 
        _idreport, 
        _idsmlouvy, 
        _idpolsml
    );

    return NEW;
end;  
$BODY$
  LANGUAGE 'plpgsql' VOLATILE
  COST 100;
ALTER FUNCTION dactiviticreatetask() OWNER TO %firmaRole%;

CREATE TRIGGER dactiviticreatetask
  AFTER INSERT
  ON act_ru_task
  FOR EACH ROW
  EXECUTE PROCEDURE dactiviticreatetask();
  
CREATE OR REPLACE FUNCTION dactivitiupdatetask()
  RETURNS trigger AS
$BODY$  
begin
    UPDATE audalosti set 
        termin = date_trunc('minute', NEW.due_date_), 
        zahajeni = date_trunc('minute', NEW.create_time_),
        iduzivatel = (select iduzivatel from wuzivatel where kod = NEW.assignee_ limit 1),
        taskDefinitionKey = NEW.task_def_key_,
        processDefinitionId = NEW.proc_def_id_
    where actRuTask = NEW.id_ and druhudalk = 'druhUdal.workflow';

return NEW;
end;  
$BODY$
  LANGUAGE 'plpgsql' VOLATILE
  COST 100;
ALTER FUNCTION dactivitiupdatetask() OWNER TO %firmaRole%;

CREATE TRIGGER dactivitiupdatetask
  AFTER UPDATE
  ON act_ru_task
  FOR EACH ROW
  EXECUTE PROCEDURE dactivitiupdatetask();

CREATE OR REPLACE FUNCTION dactivitideletetask()
  RETURNS trigger AS
$BODY$  
begin
    DELETE FROM audalosti 
    where actRuTask = OLD.id_ and druhudalk = 'druhUdal.workflow';
    
    return OLD;
end;  
$BODY$
  LANGUAGE 'plpgsql' VOLATILE
  COST 100;
ALTER FUNCTION dactivitideletetask() OWNER TO %firmaRole%;

CREATE TRIGGER dactivitideletetask
  AFTER DELETE
  ON act_ru_task
  FOR EACH ROW
  EXECUTE PROCEDURE dactivitideletetask();


-- nastaveni prav
ALTER TABLE public.ACT_GE_PROPERTY OWNER TO %firmaRole%;
ALTER TABLE public.ACT_GE_BYTEARRAY OWNER TO %firmaRole%;
ALTER TABLE public.ACT_RE_DEPLOYMENT OWNER TO %firmaRole%;
ALTER TABLE public.ACT_RE_MODEL OWNER TO %firmaRole%;
ALTER TABLE public.ACT_RU_EXECUTION OWNER TO %firmaRole%;
ALTER TABLE public.ACT_RU_JOB OWNER TO %firmaRole%;
ALTER TABLE public.ACT_RE_PROCDEF OWNER TO %firmaRole%;
ALTER TABLE public.ACT_RU_TASK OWNER TO %firmaRole%;
ALTER TABLE public.ACT_RU_IDENTITYLINK OWNER TO %firmaRole%;
ALTER TABLE public.ACT_RU_VARIABLE OWNER TO %firmaRole%;
ALTER TABLE public.ACT_RU_EVENT_SUBSCR OWNER TO %firmaRole%;

ALTER TABLE public.ACT_HI_PROCINST OWNER TO %firmaRole%;
ALTER TABLE public.ACT_HI_ACTINST OWNER TO %firmaRole%;
ALTER TABLE public.ACT_HI_TASKINST OWNER TO %firmaRole%;
ALTER TABLE public.ACT_HI_VARINST OWNER TO %firmaRole%;
ALTER TABLE public.ACT_HI_DETAIL OWNER TO %firmaRole%;
ALTER TABLE public.ACT_HI_COMMENT OWNER TO %firmaRole%;
ALTER TABLE public.ACT_HI_ATTACHMENT OWNER TO %firmaRole%;

ALTER TABLE public.ACT_ID_GROUP OWNER TO %firmaRole%;
ALTER TABLE public.ACT_ID_MEMBERSHIP OWNER TO %firmaRole%;
ALTER TABLE public.ACT_ID_USER OWNER TO %firmaRole%;
ALTER TABLE public.ACT_ID_INFO OWNER TO %firmaRole%;


ALTER TABLE audalosti
  ADD COLUMN processDefinitionId character varying(63),
  ADD COLUMN taskDefinitionKey character varying(255),
  ADD COLUMN actRuTask character varying(63);

ALTER TABLE audalosti
  ADD CONSTRAINT audalosti_processDefinitionId FOREIGN KEY (processDefinitionId)
      REFERENCES act_re_procdef (id_) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION;

CREATE INDEX audalosti_actRuTask_index ON audalosti USING btree (actRuTask);
CREATE INDEX audalosti_taskDefinitionKey_index ON audalosti USING btree (taskDefinitionKey);
CREATE INDEX audalosti_processDefinitionId_index ON audalosti USING btree (processDefinitionId);


-- zvyseni verze databaze
INSERT INTO dbversion(iddbversion, dbversion, needreimport, appversion, upgradetime)
  VALUES (nextval('seq_dbversion'), 738, 't', %appVersion%, clock_timestamp());
