--------------------------------------------------------------------------------
--  Bug 37257 – [FlexiBee] Chybějící změny ve WebHooks / ChangeLog triggery DEFERRED
--------------------------------------------------------------------------------

-- funkce pro dočasné uložení externích indentifikátorů při výmazu
CREATE OR REPLACE FUNCTION saveextid() RETURNS trigger AS $$

    spi_exec "SELECT pg_attribute.attname AS pk
        FROM pg_index, pg_class, pg_attribute
        WHERE
          pg_class.oid = '[ quote $TG_table_name ]'::regclass AND
          indrelid = pg_class.oid AND
          pg_attribute.attrelid = pg_class.oid AND
          pg_attribute.attnum = any(pg_index.indkey) AND indisprimary"

    set pkval [ quote $OLD([ quote $pk ]) ]
    set extid_table "changelog_extid_${TG_table_name}_pk${pkval}"
    
    spi_exec "SELECT attname AS fk
            FROM pg_attribute
            WHERE attrelid = 'wextid'::regclass
                AND attnum IN (
                    SELECT conkey\[1\]
                    FROM pg_constraint
                    WHERE conrelid = 'wextid'::regclass
                        AND confrelid = '[ quote $TG_table_name ]'::regclass)" {
            
        spi_exec "CREATE TEMPORARY TABLE $extid_table ON COMMIT DROP AS SELECT * FROM wextid WHERE $fk = $pkval"
    }
    
    return [ array get OLD ]
    
$$ LANGUAGE pltcl VOLATILE SECURITY DEFINER;

ALTER FUNCTION saveextid() OWNER TO %admin%;


-- funkce pro okamžité vyvolání odložených (DEFERRED) triggerů nad tabulkou changelog
CREATE OR REPLACE FUNCTION invoke_deferred_changes_triggers_immediate()
RETURNS void AS $$
    spi_exec "SELECT t.tgname AS tgname FROM pg_trigger t WHERE t.tgname ~ 'changes$' AND t.tgdeferrable AND t.tginitdeferred" {
        spi_exec "SET CONSTRAINTS $tgname IMMEDIATE"
    }
$$ LANGUAGE pltcl;

ALTER FUNCTION invoke_deferred_changes_triggers_immediate() OWNER TO %firmaRole%;


-- "aktualizace" skriptu: TETA s.r.o. měla dočasně předřazeno PERFORM invoke_deferred_changes_triggers_immediate() pro DROP
CREATE OR REPLACE FUNCTION create_or_drop_changes_triggers(docreate boolean)
  RETURNS void AS
$BODY$
BEGIN
  IF docreate THEN
    PERFORM create_or_drop_ucto_changes_triggers(true);
    PERFORM create_or_drop_majetek_changes_triggers(true);
    PERFORM create_or_drop_mzdy_changes_triggers(true);
  ELSE
    PERFORM create_or_drop_ucto_changes_triggers(false);
    PERFORM create_or_drop_majetek_changes_triggers(false);
    PERFORM create_or_drop_mzdy_changes_triggers(false);
  END IF;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE SECURITY DEFINER;


-- oprava vlastníků pro changelog (existují případy, kde je to špatně)
ALTER TABLE changelog OWNER TO %admin%;
GRANT SELECT ON TABLE changelog TO %firmaRole%;


-- zvyseni verze databaze
insert into dbversion(iddbversion, dbversion, needreimport, appversion, upgradetime)
  values (nextval('seq_dbversion'), 824, 't', %appVersion%, clock_timestamp());
