----------------------------------------------------------------------------------------------
-- Bug 43123 - Není možné vybrat výrobní čísla při tvorbě dobropisu (Macourek MicroWare) 
----------------------------------------------------------------------------------------------

/* funkce, které se přes převodky pokusí dojít k dosud nevydaným výrobním číslům, která jsou navázána na idVydej ze vstupu
* @param idVydej pole idček dpolSklad, skladových položek, které znamenaly výdej výrobního čísla
* @param kody kódy výrobních čísel, která byla vydána, index v poli s kódy by měl odpovídat tomu v poli idVydej
*
* @return nalezená dosud nevydaná výrobní čísla
*/
CREATE OR REPLACE FUNCTION findVyrCislaPrijatePresPrevodky(IN idvydej integer[], IN kody text[])
    RETURNS SETOF svyrobnicislo AS
    $BODY$

    DECLARE
        t record;
        _pocet integer;

    begin

    -- pomocná tabulka
    create TEMP table tmp_ids(
        -- svyrobnicislo.idpolskladvydej
       _idVydej integer,
        --
       _kod varchar(255),
       -- svyrobnicislo.idvyrobnicislo
       _idVyrCis integer,
       -- poslední výsledek, vrátiti se mají jen ty s true
       _final boolean,
       -- pomocná proměnná (stav) 0 = ještě nezpracováno, 1 zpracovává se, 2 = zpracováno
       _stav integer

    ) ON COMMIT DROP;

    --naplnění pomocné tabulky hodnotami ze vstupu
    FOR i IN 1 .. array_upper(idvydej, 1) loop
            insert into tmp_ids(_idVydej, _kod, _idVyrCis, _final, _stav) values (idvydej[i], kody[i], null, false,0);
    end loop;


    while 
        (select count ( vc.*)
            FROM dvazebtab vt
                 inner join svyrobnicislo vc on vc.idpolskladprijem = vt.idtabb
             where vt.idtaba in (select _idVydej from tmp_ids where _idVydej is not null and not _final and _stav = 0)) > 0 
        loop

            update tmp_ids set _stav = 1 where _stav = 0;
            insert into tmp_ids(_idVydej, _idVyrCis, _final, _stav)  (
                   SELECT  vc.idpolskladvydej, vc.idvyrobnicislo, false, 0 
                    FROM dvazebtab vt
                        inner join tmp_ids tmp on tmp._idvydej = vt.idtaba
                        inner join svyrobnicislo vc on vc.idpolskladprijem = vt.idtabb
                        where not tmp._final and tmp._stav = 1 
                            and vt.typvazbyk in ('typVazbyDokl.polozkaSklPrevod', 'typVazbyDokl.polozkaDobropis') and vc.kod = tmp._kod );

                    update tmp_ids set _final = true, _stav = 2 where _idVydej is null;
                    update tmp_ids set _stav = 2 where _stav = 1;

    end loop;
    return query select * from svyrobnicislo where idvyrobnicislo in (select _idvyrcis from tmp_ids where _final);

end
$BODY$
LANGUAGE plpgsql;
ALTER FUNCTION findVyrCislaPrijatePresPrevodky(integer [], text[])
  OWNER TO %firmaRole%;

-- zvyseni verze databaze
INSERT INTO dbversion(iddbversion, dbversion, needreimport, appversion, upgradetime)
  VALUES (nextval('seq_dbversion'), 942, 't', %appVersion%, clock_timestamp());
