<?xml version="1.0" encoding="utf-8" ?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:java="http://xml.apache.org/xalan/java" exclude-result-prefixes="java">
    <!-- fakturační XML pro pokladny firmy APLS -->
    <xsl:output method="xml"  encoding="utf-8" />

    <xsl:template match="winstrom">
	<winstrom version="1.0">
		<xsl:apply-templates select="faktura-vydana"/>
	</winstrom>
    </xsl:template>

    <xsl:template name="adresar">
			<ic><xsl:value-of select="ODBERATEL_ICO"/></ic>
			<dic><xsl:value-of select="ODBERATEL_DIC"/></dic>

			<nazev><xsl:value-of select="ODBERATEL_NAZEV"/></nazev>
			<ulice><xsl:value-of select="ODBERATEL_ADRESA"/></ulice>
			<mesto><xsl:value-of select="ODBERATEL_MESTO"/></mesto>
			<psc><xsl:value-of select="ODBERATEL_PSC"/></psc>
			<stat><xsl:choose>
			    <xsl:when test="ODBERATEL_ZEME = 'CR'">
                                code:CZ
			    </xsl:when>
			    <xsl:when test="ODBERATEL_ZEME = 'SR'">
                                code:SK
			    </xsl:when>
			    <xsl:otherwise>
				code:<xsl:value-of select="ODBERATEL_ZEME"/>
			    </xsl:otherwise>
			</xsl:choose>
			</stat>

    </xsl:template>
    
    <xsl:template match="faktura-vydana">
		<!-- pokud neexistuje adresář (dle IČ a DIČ, založíme jej) -->
		<adresar update="ignore">
			<xsl:if test="ODBERATEL_DIC != ''">
			    <id>vatid:<xsl:value-of select="ODBERATEL_DIC"/></id>
			</xsl:if>
			<xsl:if test="ODBERATEL_ICO != ''">
			    <id>in:<xsl:value-of select="ODBERATEL_ICO"/></id>
			</xsl:if>

		    <xsl:call-template name="adresar"/>
		</adresar>


		<faktura-vydana>
			<id>ext:APLS:<xsl:value-of select="VARIABILNI_SYMBOL"/></id>
			<typDokl>code:<xsl:value-of select="java:flexibee.Tool.toUpper(java:flexibee.Tool.params('typ-dokladu-faktury'))" /></typDokl>

			<varSym><xsl:value-of select="VARIABILNI_SYMBOL"/></varSym>
			<datVyst><xsl:value-of select="java:flexibee.Tool.parseDate(VYSTAVENO, 'd.m.y')" /></datVyst>
			<duzpPuv><xsl:value-of select="java:flexibee.Tool.parseDate(DUZP, 'd.m.y')" /></duzpPuv>
			<datSplat><xsl:value-of select="java:flexibee.Tool.parseDate(SPLATNO, 'd.m.y')" /></datSplat>

			<bezPolozek>true</bezPolozek>
			<sumOsv><xsl:value-of select="ZAKLAD_0"/></sumOsv>
			<sumZklSniz><xsl:value-of select="ZAKLAD_SS"/></sumZklSniz>
			<sumZklZakl><xsl:value-of select="ZAKLAD_ZS"/></sumZklZakl>
			<sumDphSniz><xsl:value-of select="DAN_SS"/></sumDphSniz>
			<sumDphZakl><xsl:value-of select="DAN_ZS"/></sumDphZakl>
			<sumCelkem><xsl:value-of select="DAN_ZS"/></sumCelkem>


			<firma><xsl:if test="ODBERATEL_DIC != ''">[vatid:<xsl:value-of select="ODBERATEL_DIC"/>]</xsl:if><xsl:if test="ODBERATEL_ICO != ''">[in:<xsl:value-of select="ODBERATEL_ICO"/>]</xsl:if></firma>


			<xsl:if test="java:flexibee.Tool.trim(CIZI_MENA) != ''">
			    <mena>code:<xsl:value-of select="CIZI_MENA"/></mena>
			    <kurz><xsl:value-of select="CIZI_KURZ"/></kurz>
			</xsl:if>

			<stredisko>code:<xsl:value-of select="STREDISKO"/></stredisko>

			<xsl:choose>
			    <xsl:when test="ZPUSOB_UHRADY = '1'">
				<formaUhrK>formaUhr.prevod</formaUhrK>
			    </xsl:when>
			    <xsl:otherwise>
				<formaUhrK>formaUhr.prevod</formaUhrK>
			    </xsl:otherwise>
			</xsl:choose>

			<xsl:call-template name="adresar"/>


		</faktura-vydana>
    </xsl:template>

<!--

    <VARIABILNI_SYMBOL>201000</VARIABILNI_SYMBOL>	-ok
    <DUZP>1.1.2010</DUZP>				-ok
    <VYSTAVENO>1.1.2010</VYSTAVENO>			-ok
    <SPLATNO>10.1.2010</SPLATNO>			-ok
    <ZAKLAD_0>0.00</ZAKLAD_0>				-ok
    <ZAKLAD_SS>0.00</ZAKLAD_SS>				-ok
    <ZAKLAD_ZS>10000.00</ZAKLAD_ZS>			-ok
    <DAN_SS>0.00</DAN_SS>				-ok
    <DAN_ZS>2000.00</DAN_ZS>				-ok
    <CELKEM>12000.00</CELKEM>				-ok
    <CIZI_KURZ>0.0000</CIZI_KURZ>			-ok
    <CIZI_CELKEM>0.000</CIZI_CELKEM>
    <STREDISKO>100</STREDISKO>				-ok
    <ZPUSOB_UHRADY>1</ZPUSOB_UHRADY>			-ok
    <ODBERATEL_DIC>CZ01010101</ODBERATEL_DIC>		-ok
    <ODBERATEL_ICO>1010101</ODBERATEL_ICO>		-ok
    <ODBERATEL_NAZEV>Karel Novotný s.r.o.</ODBERATEL_NAZEV> -ok
    <ODBERATEL_ADRESA>Dlouhá 24</ODBERATEL_ADRESA>	-ok
    <ODBERATEL_MESTO>Praha</ODBERATEL_MESTO>		-ok
    <ODBERATEL_PSC>12000</ODBERATEL_PSC>		-ok
    <ODBERATEL_ZEME>CR</ODBERATEL_ZEME>			-ok
    <ZALOHA_ZAKLAD_SS>0.00</ZALOHA_ZAKLAD_SS>
    <ZALOHA_ZAKLAD_ZS>0.00</ZALOHA_ZAKLAD_ZS>
    <ZALOHA_DAN_SS>0.00</ZALOHA_DAN_SS>
    <ZALOHA_DAN_ZS>0.00</ZALOHA_DAN_ZS>


-->

</xsl:stylesheet>