<?xml version="1.0" encoding="utf-8" ?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:java="http://xml.apache.org/xalan/java" exclude-result-prefixes="java">
  <xsl:output method="xml" indent="yes" encoding="utf-8" />
  <xsl:template match="MoneyData">
    <winstrom version="1.0" source="s3">
      <!-- kontrola a pripadne zalozeni dobropisu -->
      <xsl:if test="./SeznamFaktVyd/FaktVyd[Dobropis>0]">
        <typ-faktury-vydane update="ignore">
          <id>ext:mS3tfav:MS3DOBROPIS</id>
          <id>code:MS3DOBROPIS</id>
          <kod>MS3DOBROPIS</kod>
          <nazev>Dobropis - nedaňový doklad pro doklady z Money S3</nazev>
          <modul>FAV</modul>
          <ucetni>false</ucetni>
          <typDoklK>typDokladu.dobropis</typDoklK>
        </typ-faktury-vydane>
      </xsl:if>
      
      <!-- kontrola a pripadne zalozeni proforma -->
      <xsl:if test="./SeznamFaktVyd/FaktVyd[Druh='F']">
        <typ-faktury-vydane update="ignore">
          <id>ext:mS3tfav:MS3PROFORMA</id>
          <id>code:MS3PROFORMA</id>
          <kod>MS3PROFORMA</kod>
          <nazev>Proforma - nedaňový doklad pro doklady z Money S3</nazev>
          <modul>FAV</modul>
          <ucetni>false</ucetni>
          <typDoklK>typDokladu.proforma</typDoklK>
        </typ-faktury-vydane>
      </xsl:if>
      
      <!-- kontrola a pripadne zalozeni zalohy -->
      <xsl:if test="./SeznamFaktVyd/FaktVyd[Druh='L']">
        <typ-faktury-vydane update="ignore">
          <id>ext:mS3tfav:MS3ZALOHA</id>
          <id>code:MS3ZALOHA</id>
          <kod>MS3ZALOHA</kod>
          <nazev>Záloha - nedaňový doklad pro doklady z Money S3</nazev>
          <modul>FAV</modul>
          <ucetni>false</ucetni>
          <typDoklK>typDokladu.zalohFaktura</typDoklK>
        </typ-faktury-vydane>
      </xsl:if>
      
      <!-- zalozeni typ dokladu zbylych fav -->
      <typ-faktury-vydane update="ignore">
          <id>ext:mS3tfav:MS3FAKTURA</id>
          <id>code:MS3FAKTURA</id>
          <kod>MS3FAKTURA</kod>
          <nazev>Faktura - nedaňový doklad pro doklady z Money S3</nazev>
          <modul>FAV</modul>
          <ucetni>false</ucetni>
          <typDoklK>typDokladu.faktura</typDoklK>
      </typ-faktury-vydane>
      
      <xsl:apply-templates select="SeznamFaktVyd"/>
    </winstrom>
  </xsl:template>
  
<!-- template pro spolecne veci jak vydanych tak prijatych faktur -->
  <xsl:template name="dokladCommon">
    <kod><xsl:value-of select="Doklad"/></kod>
    <popis><xsl:value-of select="Popis"/></popis>
    <datVyst><xsl:value-of select="java:flexibee.Tool.parseDate(Vystaveno, 'y-M-d')"/></datVyst>
    <datSplat><xsl:value-of select="java:flexibee.Tool.parseDate(Splatno, 'y-M-d')"/></datSplat>
    <duzpPuv><xsl:value-of select="java:flexibee.Tool.parseDate(PlnenoDPH, 'y-M-d')"/></duzpPuv>
    <duzpUcto><xsl:value-of select="java:flexibee.Tool.parseDate(PlnenoDPH, 'y-M-d')"/></duzpUcto>
    
    <xsl:if test="KonstSym">
      <konSym if-not-found="nearest-invalid">code:<xsl:value-of select="java:flexibee.Tool.padNumber(4,KonstSym)"/></konSym>
    </xsl:if>
    <varSym><xsl:value-of select="VarSymbol"/></varSym>

    <formaUhrK>
    <xsl:choose><xsl:when test="Uhrada = 'převodem'">formaUhr.prevod</xsl:when>
    <xsl:otherwise>formaUhr.nespec</xsl:otherwise>
    </xsl:choose>
    </formaUhrK>
    
    <szbDphSniz><xsl:value-of select="java:flexibee.Tool.parseMoney(SazbaDPH1)"/></szbDphSniz>
    <szbDphZakl><xsl:value-of select="java:flexibee.Tool.parseMoney(SazbaDPH2)"/></szbDphZakl>
    
    <xsl:comment>castky</xsl:comment>
    <!-- castky jen u bezpolozkoveho dokladu -->
    <xsl:if test="not(./Valuty)">
      <sumOsv><xsl:value-of select="./SouhrnDPH/Zaklad0"/></sumOsv>
      <sumZklSniz><xsl:value-of select="./SouhrnDPH/Zaklad5"/></sumZklSniz>
      <sumZklZakl><xsl:value-of select="./SouhrnDPH/Zaklad22"/></sumZklZakl>
      <sumDphSniz><xsl:value-of select="./SouhrnDPH/DPH5"/></sumDphSniz>
      <sumDphZakl><xsl:value-of select="./SouhrnDPH/DPH22"/></sumDphZakl>
    </xsl:if>
  </xsl:template>
  
  <xsl:template name="dodOdb">
    <xsl:if test="./DodOdb/GUID">
      <firma>ext:mS3adr:<xsl:value-of select="substring(./DodOdb/GUID, 2, string-length(./DodOdb/GUID)-2)"/></firma>
    </xsl:if>
    <nazFirmy><xsl:value-of select="./DodOdb/FaktNazev"/></nazFirmy>
    <ulice><xsl:value-of select="./DodOdb/FaktAdresa/Ulice"/></ulice>
    <mesto><xsl:value-of select="./DodOdb/FaktAdresa/Misto"/></mesto>
    <psc><xsl:value-of select="./DodOdb/FaktAdresa/PSC"/></psc>
    <ic><xsl:value-of select="./DodOdb/ICO"/></ic>
    <dic><xsl:value-of select="./DodOdb/DIC"/></dic>

    <buc><xsl:value-of select="./DodOdb/Ucet"/></buc>
    <xsl:if test="./DodOdb/KodBanky != '' and java:flexibee.MigrationTool.existKodable('smerKod', ./DodOdb/KodBanky)">
        <smerKod>code:<xsl:value-of select="./DodOdb/KodBanky"/></smerKod>
    </xsl:if>
    <banSpojDod if-not-found="null">ext:mS3bu:<xsl:value-of select="concat(substring(./DodOdb/GUID, 2, string-length(./DodOdb/GUID)-2), ./DodOdb/KodBanky)"/></banSpojDod>
  </xsl:template>
  
<!-- vydane faktury -->
  <xsl:template name="polozkyToPozn">
    <poznam>
      <xsl:for-each select="./SeznamPolozek/Polozka">
          <xsl:value-of select="concat(Popis, ' kusu:', PocetMJ, ' za:', Cena * PocetMJ, ' mena:', Valuty * PocetMJ, ';')"/>
      </xsl:for-each>
    </poznam>
  </xsl:template>

  <xsl:comment>vydane faktury</xsl:comment>
  <xsl:template match="SeznamFaktVyd">
    <xsl:apply-templates select="FaktVyd"/>
  </xsl:template>

  <xsl:template match="FaktVyd">
    <xsl:if test="./Doklad">
    <faktura-vydana>
      <id>ext:mS3fav:<xsl:value-of select="Doklad"/></id>
      <xsl:call-template name="dokladCommon"/>
      
      <typDokl>
        <xsl:choose>
          <xsl:when test="Dobropis = '1'">code:MS3DOBROPIS</xsl:when>
          <xsl:when test="Druh = 'F'">code:MS3PROFORMA</xsl:when>
          <xsl:when test="Druh = 'L'">code:MS3ZALOHA</xsl:when>
          <xsl:when test="Druh = 'N'">code:MS3FAKTURA</xsl:when>
          <xsl:otherwise>code:MS3FAKTURA</xsl:otherwise>
        </xsl:choose>
      </typDokl>
      
      <!-- je-li doklad v mene -->
      <xsl:if test="./Valuty">
        <mena>code:<xsl:value-of select="java:flexibee.Tool.toUpper(./Valuty/Mena/Kod)"/></mena>
        <kurz><xsl:value-of select="./Valuty/Mena/Kurs"/></kurz>
        <kurzMnozstvi><xsl:value-of select="./Valuty/Mena/Mnozstvi"/></kurzMnozstvi>

        <sumOsvMen><xsl:value-of select="./Valuty/SouhrnDPH/Zaklad0"/></sumOsvMen>
        <sumZklSnizMen><xsl:value-of select="./Valuty/SouhrnDPH/Zaklad5"/></sumZklSnizMen>
        <sumZklZaklMen><xsl:value-of select="./Valuty/SouhrnDPH/Zaklad22"/></sumZklZaklMen>
        <sumDphSnizMen><xsl:value-of select="./Valuty/SouhrnDPH/DPH5"/></sumDphSnizMen>
        <sumDphZaklMen><xsl:value-of select="./Valuty/SouhrnDPH/DPH22"/></sumDphZaklMen>
      </xsl:if>
      
      <zbyvaUhradit><xsl:value-of select="PriUhrZbyv"/></zbyvaUhradit>
      <cisObj><xsl:value-of select="substring(CObjednavk, 1, 30)"/></cisObj>
      <xsl:comment>odberatel</xsl:comment>
      <!-- odberatel -->
      <xsl:if test="./DodOdb">
        <xsl:call-template name="dodOdb"/>
      </xsl:if>
      
      <xsl:comment>polozky faktury</xsl:comment>
      <!-- polozky faktury jenom jako poznamka -->
      <xsl:if test="./SeznamPolozek">
        <xsl:call-template name="polozkyToPozn"/>
      </xsl:if>
      <bezPolozek>true</bezPolozek>
    </faktura-vydana>
    </xsl:if>
  </xsl:template>
</xsl:stylesheet>
